@@ -29,8 +29,10 @@ except ImportError:
 import gi
 
 import cairo
-import mlt7 as mlt
-import time
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 
 from gi.repository import Gtk, GObject
 from gi.repository import GLib
@@ -46,6 +48,7 @@ import editorstate
 import mltrefhold
 import guiutils
 import utils
+import utilsgtk
 
 SLOT_W = 60
 METER_SLOT_H = 458
@@ -61,7 +64,7 @@ METER_LIGHTS = 143 #57
 METER_HEIGHT = METER_LIGHTS * DASH_INK + (METER_LIGHTS - 1) * DASH_SKIP
 METER_WIDTH = 10
 
-# These are calculated using IEC_Scale function in MLT and correspond to level values received here
+# These are calculated using IEC_Scale function in MLT and correspond to level values received here.
 DB_IEC_MINUS_2 = 0.95
 DB_IEC_MINUS_4 = 0.9
 DB_IEC_MINUS_6 = 0.85
@@ -75,7 +78,8 @@ OVER_FRAMES = 30
 
 # Colors
 METER_BG_COLOR = (0.15, 0.15, 0.15)
-OVERLAY_COLOR = (0.70, 0.70, 0.70) #utils.get_cairo_color_tuple_255_rgb(63, 145, 188)#59, 140, 174) #(0.7, 0.7, 1.0)
+OVERLAY_COLOR = (0.70, 0.70, 0.70)
+CHANNEL_ID_COLOR = (0.06, 0.34, 0.66)
 
 # Color gradient used to draw "LED" colors
 rr, rg, rb = utils.get_cairo_color_tuple_255_rgb(219, 69, 69)
@@ -124,22 +128,26 @@ def init(profile):
             METER_LIGHTS = 82
             METER_HEIGHT = METER_LIGHTS * DASH_INK + (METER_LIGHTS - 1) * DASH_SKIP
 
+    # We want a global ticker object to always exist, so we can use it to know if it is running.
+    launch_update_ticker()
+    _update_ticker.destroy_ticker()
+            
+def launch_update_ticker():
     # We want this to be always present when closing app or we'll need to handle it being missing.
     global _update_ticker
-    _update_ticker = utils.Ticker(_audio_monitor_update, 0.04)
+    _update_ticker = utilsgtk.GtkTicker(_audio_monitor_update, 40)
     _update_ticker.start_ticker()
-    _update_ticker.stop_ticker()    
-
+  
 def init_for_project_load():
     # Monitor window is quaranteed to be closed
     if _update_ticker.running:
-        _update_ticker.stop_ticker()    
+        _update_ticker.destroy_ticker()    
         
     global _level_filters
     _level_filters = None
     _init_level_filters(False)
 
-    _update_ticker.start_ticker()
+    launch_update_ticker()
 
 def update_mute_states():
     if _monitor_window != None:
@@ -148,7 +156,7 @@ def update_mute_states():
 def close():
     close_audio_monitor()
     close_master_meter()
-    _update_ticker.stop_ticker()
+    _update_ticker.destroy_ticker()
 
 def show_audio_monitor():
     global _monitor_window
@@ -161,7 +169,7 @@ def show_audio_monitor():
         
     global _update_ticker
     if _update_ticker.running == False:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
 def close_audio_monitor():
     
@@ -195,7 +203,7 @@ def get_master_meter():
     _master_volume_meter = MasterVolumeMeter()
 
     if _update_ticker.running == False:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
     align = guiutils.set_margins(_master_volume_meter.widget, 3, 3, 3, 3)
     
@@ -258,7 +266,7 @@ def _destroy_level_filters(destroy_track_filters=False):
 
     # We need to be sure that audio level updates are stopped before
     # detaching and destroying them
-    _update_ticker.stop_ticker()
+    _update_ticker.destroy_ticker()
 
     # Detach filters
     if len(_level_filters) != 0:
@@ -278,17 +286,17 @@ def _destroy_level_filters(destroy_track_filters=False):
         _audio_levels = []
     elif _monitor_window == None:
         _level_filters = [_level_filters[0]]
-        _audio_levels[0] = 0.0
+        _audio_levels[0] = (0.0, 0.0)
 
     if _master_volume_meter != None or _monitor_window != None:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
 def recreate_master_meter_filter_for_new_sequence():
     global _level_filters, _audio_levels
 
     # We need to be sure that audio level updates are stopped before
     # detaching and destroying them
-    _update_ticker.stop_ticker()
+    _update_ticker.destroy_ticker()
 
     if len(_level_filters) != 0:
         seq = editorstate.current_sequence()
@@ -296,12 +304,12 @@ def recreate_master_meter_filter_for_new_sequence():
         if _master_volume_meter != None:
             seq.tractor.detach(_level_filters[0])
             _level_filters.pop(0)
-            _audio_levels[0] = 0.0
+            _audio_levels[0] = (0.0, 0.0)
             master_level_filter = _add_audio_level_filter(seq.tractor, seq.profile)
             _level_filters.insert(0, master_level_filter)
 
     if _master_volume_meter != None or _monitor_window != None:
-        _update_ticker.start_ticker()
+        launch_update_ticker()
 
 def _add_audio_level_filter(producer, profile):
     audio_level_filter = mlt.Filter(profile, "audiolevel")
@@ -309,14 +317,12 @@ def _add_audio_level_filter(producer, profile):
     producer.attach(audio_level_filter)
     return audio_level_filter
 
-def _audio_monitor_update():
-    # This is not called from gtk thread
-
+def _audio_monitor_update(data):
+    # 'data' is not used here. 
+    # This is called holding Gdk lock so we can do Gtk updates.
     if _monitor_window == None and _master_volume_meter == None:
         return
 
-    Gdk.threads_enter()
-
     global _audio_levels
     _audio_levels = []
     for i in range(0, len(_level_filters)):
@@ -330,8 +336,6 @@ def _audio_monitor_update():
     if _master_volume_meter != None:
         _master_volume_meter.canvas.queue_draw()
 
-    Gdk.threads_leave()
-
 def _get_channel_value(audio_level_filter, channel_property):
     level_value = audio_level_filter.get(channel_property)
     if level_value == None:
@@ -540,7 +544,7 @@ class ChannelMeter:
         self.draw_text(val_text, "Sans 8", cr, x + self.dB_x_pad + x_fine_tune, y - 8 + self.y_top_pad, OVERLAY_COLOR)
         
     def draw_channel_identifier(self, cr, x):
-        self.draw_text(self.channel_text, "Sans Bold 8", cr, x - 4, self.height + 2 +  self.y_top_pad, OVERLAY_COLOR)
+        self.draw_text(self.channel_text, "Sans Bold 8", cr, x - 4, self.height + 2 +  self.y_top_pad, CHANNEL_ID_COLOR)
 
     def draw_text(self, text, font_desc, cr, x, y, color):
         layout = PangoCairo.create_layout(cr)
@@ -570,7 +574,7 @@ class GainControl(Gtk.Frame):
             gain_value = producer.audio_gain # track
         gain_value = gain_value * 100
         
-        self.adjustment = Gtk.Adjustment(value=gain_value, lower=0, upper=100, step_incr=1)
+        self.adjustment = Gtk.Adjustment(value=gain_value, lower=0, upper=100, step_increment=1)
         self.slider = Gtk.VScale()
         self.slider.set_adjustment(self.adjustment)
         self.slider.set_size_request(SLOT_W - 10, CONTROL_SLOT_H - 105)
@@ -585,7 +589,7 @@ class GainControl(Gtk.Frame):
             pan_value = 0.5 # center
         pan_value = (pan_value - 0.5) * 200 # from range 0 - 1 to range -100 - 100
 
-        self.pan_adjustment = Gtk.Adjustment(value=pan_value, lower=-100, upper=100, step_incr=1)
+        self.pan_adjustment = Gtk.Adjustment(value=pan_value, lower=-100, upper=100, step_increment=1)
         self.pan_slider = Gtk.HScale()
         self.pan_slider.set_adjustment(self.pan_adjustment)
         self.pan_slider.connect("value-changed", self.pan_changed)
@@ -594,12 +598,6 @@ class GainControl(Gtk.Frame):
         surface = guiutils.get_cairo_image("pan_track")
         self.pan_button.set_image(Gtk.Image.new_from_surface (surface))
         self.pan_button.connect("toggled", self.pan_active_toggled)
-        
-        if pan_value == 0.0:
-            self.pan_slider.set_sensitive(False)
-        else:
-            self.pan_button.set_active(True)
-            self.pan_adjustment.set_value(pan_value) # setting button active sets value = 0, set correct value again
 
         label = guiutils.bold_label(name)
 
@@ -617,6 +615,12 @@ class GainControl(Gtk.Frame):
 
         self.mute_changed()
 
+        if pan_value == 0.0:
+            self.pan_slider.set_sensitive(False)
+        else:
+            self.pan_button.set_active(True)
+            self.pan_adjustment.set_value(pan_value) # setting button active sets value = 0, set correct value again
+            
     def gain_changed(self, slider):
         gain = slider.get_value() / 100.0
         if self.is_master == True:
@@ -705,5 +709,6 @@ class MasterVolumeMeter:
         grad.add_color_stop_rgba(*GREEN_2)
 
         l_value, r_value = _audio_levels[0]
+
         x = 0
         self.meter.display_value(cr, x, l_value, r_value, grad)
