@@ -27,13 +27,15 @@ from PIL import Image
 from gi.repository import GLib
 from gi.repository import Gtk
 
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 from operator import itemgetter
 import os
 import shutil
 import time
 
-import audiowaveform
 import audiosync
 import appconsts
 import clipeffectseditor
@@ -44,15 +46,16 @@ import dialogutils
 import gui
 import guicomponents
 import edit
-import editevent
 from editorstate import current_sequence
 from editorstate import get_track
 from editorstate import PROJECT
 from editorstate import PLAYER
 import kftoolmode
+import mediaplugin
 import mlttransitions
 import modesetting
 import movemodes
+import projectaction
 import syncsplitevent
 import tlinewidgets
 import tlineaction
@@ -76,16 +79,23 @@ def display_clip_menu(y, event, frame):
         return False
     
     # Display popup
+    not_multi_selection = (movemodes.selected_range_in == -1 or (movemodes.selected_range_out - movemodes.selected_range_in) == 0)
     gui.tline_canvas.drag_on = False
     pressed_clip = track.clips[clip_index]
     if pressed_clip.is_blanck_clip == False:
-        movemodes.select_clip(track.id, clip_index)
+        if not_multi_selection == True:
+            movemodes.select_clip(track.id, clip_index)
     else:
         movemodes.select_blank_range(track, pressed_clip)
 
     if track.type == appconsts.VIDEO:
-        guicomponents.display_clip_popup_menu(event, pressed_clip, \
+        if not_multi_selection == True:
+            guicomponents.display_clip_popup_menu(event, pressed_clip, \
                                               track, _clip_menu_item_activated)
+        else:
+            guicomponents.display_multi_clip_popup_menu(event, pressed_clip, \
+                                                track, _clip_menu_item_activated)
+                                              
     elif track.type == appconsts.AUDIO:
         guicomponents.display_audio_clip_popup_menu(event, pressed_clip, \
                                                     track, _clip_menu_item_activated)
@@ -120,7 +130,19 @@ def _delete_compositors(data):
         data = {"compositor":compositor}
         action = edit.delete_compositor_action(data)
         action.do_edit()
-    
+        
+def _multi_delete_compositors(data):
+    clip, track, item_id, x = data
+    for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+        composited_clip = track.clips[clip_index]
+        if composited_clip.is_blanck_clip == True:
+            continue
+        compositors = current_sequence().get_clip_compositors(composited_clip)
+        for compositor in compositors:
+            data = {"compositor":compositor}
+            action = edit.delete_compositor_action(data)
+            action.do_edit()
+
 def _open_clip_in_effects_editor(data):
     updater.open_clip_in_effects_editor(data)
     
@@ -128,6 +150,9 @@ def _open_clip_in_clip_monitor(data):
     clip, track, item_id, x = data
     
     media_file = PROJECT().get_media_file_for_path(clip.path)
+    if media_file == None and clip.container_data != None:
+        media_file = PROJECT().get_media_file_for_container_data(clip.container_data)
+
     media_file.mark_in = clip.clip_in
     media_file.mark_out = clip.clip_out
     updater.set_and_display_monitor_media_file(media_file)
@@ -221,25 +246,67 @@ def _add_filter(data):
     clipeffectseditor.set_clip(clip, track, index)
     clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
 
+def _add_filter_multi(data):
+    clip, track, item_id, item_data = data
+    x, filter_info = item_data
+    
+    clipeffectseditor.set_stack_update_blocked() # We update stack on set_clip below
+    
+    range_in = movemodes.selected_range_in
+    range_out = movemodes.selected_range_out
+    clips = track.clips[range_in:range_out +1]
+    data = {"clips":clips, 
+            "filter_info":filter_info,
+            "filter_edit_done_func":clipeffectseditor.filter_edit_multi_done_stack_update}
+    action = edit.add_filter_multi_action(data)
+    action.do_edit()
+    
+    clipeffectseditor.set_stack_update_unblocked()
+
+    # (re)open clip in editor
+    frame = tlinewidgets.get_frame(x)
+    index = track.get_clip_index_at(frame)
+    clipeffectseditor.set_clip(clip, track, index)
+    clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
+
 def _add_compositor(data):
     clip, track, item_id, item_data = data
-    x, compositor_type = item_data
-
-    frame = tlinewidgets.get_frame(x)
-    clip_index = track.get_clip_index_at(frame)
+    x, compositor_type, add_compositors_is_multi_selection = item_data
 
     target_track_index = track.id - 1
 
-    if current_sequence().compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW:
-        target_track_index = current_sequence().first_video_index
-
-    compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
-    clip_length = clip.clip_out - clip.clip_in
-    compositor_out = compositor_in + clip_length
-
     a_track = target_track_index
     b_track = track.id
 
+    if add_compositors_is_multi_selection == True:
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            composited_clip = track.clips[clip_index]
+            if composited_clip.is_blanck_clip == True:
+                continue
+            compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+            clip_length = composited_clip.clip_out - composited_clip.clip_in
+            compositor_out = compositor_in + clip_length
+            
+            edit_data = {"origin_clip_id":composited_clip.id,
+                        "in_frame":compositor_in,
+                        "out_frame":compositor_out,
+                        "a_track":a_track,
+                        "b_track":b_track,
+                        "compositor_type":compositor_type,
+                        "clip":composited_clip}
+            action = edit.add_compositor_action(edit_data)
+            action.do_edit()
+        
+        updater.repaint_tline()
+        return
+
+    frame = tlinewidgets.get_frame(x)
+    clip_index = track.get_clip_index_at(frame)
+    
+    compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+    clip_length = clip.clip_out - clip.clip_in
+    compositor_out = compositor_in + clip_length
+    
     edit_data = {"origin_clip_id":clip.id,
                 "in_frame":compositor_in,
                 "out_frame":compositor_out,
@@ -249,20 +316,48 @@ def _add_compositor(data):
                 "clip":clip}
     action = edit.add_compositor_action(edit_data)
     action.do_edit()
-    
+        
     updater.repaint_tline()
 
 def _add_autofade(data):
-    # NOTE: These stay synced only in "Top Down Auto Follow" mode, see: edit.get_full_compositor_sync_data()
+    # NOTE: These stay synhced only in "Top Down Auto Follow" mode, see: edit.get_full_compositor_sync_data()
     
     clip, track, item_id, item_data = data
-    x, compositor_type = item_data
+    x, compositor_type, add_compositors_is_multi_selection = item_data
 
     frame = tlinewidgets.get_frame(x)
     clip_index = track.get_clip_index_at(frame)
 
     target_track_index = track.id - 1
 
+    if add_compositors_is_multi_selection == True:
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            composited_clip = track.clips[clip_index]
+            if composited_clip.is_blanck_clip == True:
+                continue
+                
+            clip_length = composited_clip.clip_out - composited_clip.clip_in
+            if compositor_type == "##auto_fade_in":
+                compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+                compositor_out = compositor_in + int(utils.fps()) - 1
+            else: # fade out
+                clip_start = current_sequence().tracks[track.id].clip_start(clip_index)
+                compositor_out = clip_start + clip_length
+                compositor_in = compositor_out - int(utils.fps()) + 1
+
+            edit_data = {"origin_clip_id":composited_clip.id,
+                        "in_frame":compositor_in,
+                        "out_frame":compositor_out,
+                        "a_track":target_track_index,
+                        "b_track":track.id,
+                        "compositor_type":compositor_type,
+                        "clip":composited_clip}
+            action = edit.add_compositor_action(edit_data)
+            action.do_edit()
+        
+        updater.repaint_tline()
+        return
+
     clip_length = clip.clip_out - clip.clip_in
     if compositor_type == "##auto_fade_in":
         compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
@@ -279,6 +374,7 @@ def _add_autofade(data):
                 "b_track":track.id,
                 "compositor_type":compositor_type,
                 "clip":clip}
+    #print("edit_data", edit_data)
     action = edit.add_compositor_action(edit_data)
     action.do_edit()
     
@@ -309,7 +405,7 @@ def _mute_clip(data):
 
 def _delete_clip(data):
     tlineaction.splice_out_button_pressed()
-    
+
 def _lift(data):
     tlineaction.lift_button_pressed()
     
@@ -437,7 +533,8 @@ def clear_filters():
     track = get_track(movemodes.selected_track)
     clips = []
     for i in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
-        clips.append(track.clips[i])
+        if track.clips[i].is_blanck_clip == False:
+            clips.append(track.clips[i])
 
     data = {"clips":clips}
     action = edit.remove_multiple_filters_action(data)
@@ -446,14 +543,23 @@ def clear_filters():
     movemodes.clear_selected_clips()
     updater.repaint_tline()
 
-def _display_wavefrom(data):
-    audiowaveform.set_waveform_displayer_clip_from_popup(data)
-
-def _clear_waveform(data):
-    audiowaveform.clear_waveform(data)
-
 def  _clone_filters_from_next(data):
-    clip, track, item_id, item_data = data
+    clip, track, item_id, is_multi = data
+    if is_multi == True:
+        end_index = movemodes.selected_range_out
+        if end_index == len(track.clips) - 1:
+            return # clip is last clip
+        clone_clip = track.clips[end_index + 1]
+                    
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            target_clip = track.clips[clip_index]
+            if target_clip.is_blanck_clip == True:
+                continue
+
+            _do_filter_clone(target_clip, clone_clip)
+            
+        return
+        
     index = track.clips.index(clip)
     if index == len(track.clips) - 1:
         return # clip is last clip
@@ -461,7 +567,23 @@ def  _clone_filters_from_next(data):
     _do_filter_clone(clip, clone_clip)
 
 def _clone_filters_from_prev(data):
-    clip, track, item_id, item_data = data
+    clip, track, item_id, is_multi = data
+    
+    if is_multi == True:
+        start_index = movemodes.selected_range_in
+        if start_index == 0:
+            return  # clip is first clip
+        clone_clip = track.clips[start_index - 1]
+                    
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            target_clip = track.clips[clip_index]
+            if target_clip.is_blanck_clip == True:
+                continue
+
+            _do_filter_clone(target_clip, clone_clip)
+            
+        return
+        
     index = track.clips.index(clip)
     if index == 0:
         return # clip is first clip
@@ -479,6 +601,25 @@ def _clear_filters(data):
     clip, track, item_id, item_data = data
     clear_filters()
 
+def _multi_split_audio(data):
+    clip, track, item_id, item_data = data
+    clips = _get_non_blank_selected_clips(track)
+    
+    syncsplitevent.split_audio_from_clips_list(clips, track)
+
+def _multi_split_audio_synched(data):
+    clip, track, item_id, item_data = data
+    clips = _get_non_blank_selected_clips(track)
+    
+    syncsplitevent.split_audio_synched_from_clips_list(clips, track)
+
+def _get_non_blank_selected_clips(track):
+    clips = []
+    for i in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+        if track.clips[i].is_blanck_clip == False:
+            clips.append(track.clips[i])
+    return clips
+
 def _select_all_after(data):
     clip, track, item_id, item_data = data
     movemodes._select_multiple_clips(track.id, track.clips.index(clip), len(track.clips) - 1)
@@ -577,7 +718,6 @@ def _add_clip_marker(data):
     clip, track, item_id, item_data = data
     current_frame = PLAYER().current_frame()
 
-    playhead_on_popup_clip = True
     try:
         current_frame_clip_index = track.get_clip_index_at(current_frame)
         current_frame_clip = track.clips[current_frame_clip_index]
@@ -586,7 +726,6 @@ def _add_clip_marker(data):
     
     if current_frame_clip != clip:
         # Playhead is not on popup clip
-        
         return
 
     clip_start_in_tline = track.clip_start(current_frame_clip_index)
@@ -669,8 +808,10 @@ def _reload_clip_media(data):
     action.do_edit()
     
     
-    
-    
+def _create_container_clip_from_selection(data):
+    GLib.idle_add(projectaction.create_selection_compound_clip)
+
+
 # Functions to handle popup menu selections for strings 
 # set as activation messages in guicomponents.py
 # activation_message -> _handler_func
@@ -684,11 +825,10 @@ POPUP_HANDLERS = {"set_master":syncsplitevent.init_select_master_clip,
                   "split_audio_synched":syncsplitevent.split_audio_synched,
                   "resync":syncsplitevent.resync_clip,
                   "add_filter":_add_filter,
+                  "add_filter_multi":_add_filter_multi,
                   "add_compositor":_add_compositor,
                   "clear_sync_rel":syncsplitevent.clear_sync_relation,
                   "mute_clip":_mute_clip,
-                  "display_waveform":_display_wavefrom,
-                  "clear_waveform":_clear_waveform,
                   "delete_blank":_delete_blank,
                   "cover_with_prev": _cover_blank_from_prev,
                   "cover_with_next": _cover_blank_from_next,
@@ -717,9 +857,14 @@ POPUP_HANDLERS = {"set_master":syncsplitevent.init_select_master_clip,
                   "volumekf":_volume_keyframes,
                   "brightnesskf":_brightness_keyframes,
                   "delete_compositors":_delete_compositors,
+                  "multi_delete_compositors": _multi_delete_compositors,
                   "reload_media":_reload_clip_media,
+                  "create_multi_compound":_create_container_clip_from_selection,
+                  "multi_split_audio":_multi_split_audio,
+                  "multi_split_audio_synched":_multi_split_audio_synched,
                   "cc_render_full_media":containerclip.render_full_media,
                   "cc_render_clip":containerclip.render_clip_length,
                   "cc_go_to_underdered":containerclip.switch_to_unrendered_media,
                   "cc_render_settings":containerclip.set_render_settings,
-                  "cc_edit_program":containerclip.edit_program}
+                  "cc_edit_program":containerclip.edit_program,
+                  "cc_clone_generator":mediaplugin.add_media_plugin_clone}
