@@ -30,27 +30,24 @@ import time
 
 import appconsts
 import atomicfile
-import compositorfades
 import dialogs
 import dialogutils
 import gui
 import guicomponents
 import guiutils
 import edit
+import editorlayout
 import editorstate
 from editorstate import current_sequence
 from editorstate import PROJECT
-import editorpersistance
 import keyframeeditor
 import mlttransitions
 import propertyeditorbuilder
 import propertyedit
 import propertyparse
-import tlinerender
+# import tlinerender
 import utils
 
-COMPOSITOR_PANEL_LEFT_WIDTH = 160
-
 widgets = utils.EmptyClass()
 
 compositor = None # Compositor being edited.
@@ -62,17 +59,16 @@ _edit_polling_thread = None
 compositor_changed_since_last_save = False
 
 # This is updated when filter panel is displayed and cleared when removed.
-# Used to update kfeditors with external tline frame position changes
+# Used to update kfeditors with external tline frame position changes.
 keyframe_editor_widgets = []
 
-compositor_notebook_index = 3 # this is set 2 for the 2 window mode
-
-
+"""
 def shutdown_polling():
     global _edit_polling_thread
     if _edit_polling_thread != None:
         _edit_polling_thread.shutdown()
         _edit_polling_thread = None
+"""
 
 def create_widgets():
     """
@@ -80,14 +76,12 @@ def create_widgets():
     """
     widgets.compositor_info = guicomponents.CompositorInfoPanel()
     widgets.hamburger_launcher = guicomponents.HamburgerPressLaunch(_hamburger_launch_pressed)
+    widgets.hamburger_launcher.connect_launched_menu(guicomponents.clip_effects_hamburger_menu)
     guiutils.set_margins(widgets.hamburger_launcher.widget, 4, 6, 6, 0)
-    
 
-    # empty_label text has no effect on runtime behaviour, look to remove
-    # _display_compositor_edit_box() clearly overwrites anything we do here before user gets shown anything
     widgets.empty_label = Gtk.Label(label=_("No Compositor"))
 
-    # Edit area
+    # Edit area.
     widgets.value_edit_box = Gtk.VBox()
     widgets.value_edit_box.pack_start(widgets.empty_label, True, True, 0)
     widgets.value_edit_frame = Gtk.Frame()
@@ -97,7 +91,7 @@ def create_widgets():
 def get_compositor_clip_panel():
     create_widgets()
     
-    # Action row
+    # Action row.
     action_row = Gtk.HBox(False, 2)
     action_row.pack_start(widgets.hamburger_launcher.widget, False, False, 0)
     action_row.pack_start(Gtk.Label(), True, True, 0)
@@ -130,16 +124,17 @@ def set_compositor(new_compositor):
     set_enabled(True)
     _display_compositor_edit_box()
 
-    if editorpersistance.prefs.default_layout == True:
-        gui.middle_notebook.set_current_page(compositor_notebook_index)
+    editorlayout.show_panel(appconsts.PANEL_MULTI_EDIT)
 
-    global _edit_polling_thread
+    gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_COMPOSITORS)
+
+    #global _edit_polling_thread
     # Close old polling
-    if _edit_polling_thread != None:
-        _edit_polling_thread.shutdown()
+    #if _edit_polling_thread != None:
+    #    _edit_polling_thread.shutdown()
     # Start new polling
-    _edit_polling_thread = PropertyChangePollingThread()
-    _edit_polling_thread.start()
+    #_edit_polling_thread = PropertyChangePollingThread()
+    #_edit_polling_thread.start()
 
 def clear_compositor():
     global compositor
@@ -147,7 +142,7 @@ def clear_compositor():
     widgets.compositor_info.set_no_compositor_info()
     _display_compositor_edit_box()
     set_enabled(False)
-    shutdown_polling()
+    # shutdown_polling()
 
 def set_enabled(value):
     widgets.empty_label.set_sensitive(value)
@@ -238,15 +233,30 @@ def _display_compositor_edit_box():
         target_row.pack_start(Gtk.Label(label=_("Destination Track:")), False, False, 0)
         target_row.pack_start(guiutils.get_pad_label(5, 3), False, False, 0)
         target_row.pack_start(target_combo, False, False, 0)
-        target_row.pack_start(Gtk.Label(), True, True, 0)
+
         vbox.pack_start(target_row, False, False, 0)
         vbox.pack_start(guicomponents.EditorSeparator().widget, False, False, 0)
 
+    blend_added = False # for GUI prettyness.
+    
     # Transition editors
     t_editable_properties = propertyedit.get_transition_editable_properties(compositor)
     for ep in t_editable_properties:
         editor_row = propertyeditorbuilder.get_editor_row(ep)
-            
+  
+        # Add blend mode editor in top row if exists, we decided on this later to save vertical space.
+        # Using display name is bit hacky, but should work.
+        try:
+            if ep.args["displayname"] == "Blend!Mode":
+                target_row.pack_start(guiutils.get_pad_label(8, 3), False, False, 0)
+                editor_row.remove(editor_row.get_children()[0])
+                target_row.pack_start(editor_row, True, True, 0)
+                blend_added = True
+                continue
+        except:
+            pass # Not all EditablePropty objects have "displayname" arg, move on.
+    
+        # Add editor row in panel
         if editor_row != None: # Some properties don't have editors
             vbox.pack_start(editor_row, False, False, 0)
             vbox.pack_start(guicomponents.EditorSeparator().widget, False, False, 0)
@@ -262,9 +272,14 @@ def _display_compositor_edit_box():
             or (editor_type == propertyeditorbuilder.KEYFRAME_EDITOR_CLIP)
             or (editor_type == propertyeditorbuilder.KEYFRAME_EDITOR_CLIP_FADE)
             or (editor_type == propertyeditorbuilder.FADE_LENGTH)
-            or (editor_type == propertyeditorbuilder.GEOMETRY_EDITOR)):
+            or (editor_type == propertyeditorbuilder.GEOMETRY_EDITOR)
+            or (editor_type == propertyeditorbuilder.ROTATION_GEOMETRY_EDITOR_BUILDER)):
                 keyframe_editor_widgets.append(editor_row)
     
+    # Add pad label after possibly adding blend mode editor.
+    if blend_added == False:
+        target_row.pack_start(Gtk.Label(), True, True, 0)
+            
     # Extra editors. Editable properties have already been created with "editor=no_editor"
     # and will be looked up by editors from clip
     editor_rows = propertyeditorbuilder.get_transition_extra_editor_rows(compositor, t_editable_properties)
@@ -380,6 +395,7 @@ def _set_fade_length_dialog_callback(dialog, response_id, spin):
         
     dialog.destroy()
 
+"""
 class PropertyChangePollingThread(threading.Thread):
     
     def __init__(self):
@@ -424,7 +440,7 @@ class PropertyChangePollingThread(threading.Thread):
         
     def shutdown(self):
         self.running = False
-        
+"""
 
 class CompositorValuesSaveData:
     
