@@ -21,16 +21,15 @@
 """
 Module contains utility methods for creating GUI objects.
 """
-
+import cairo
 import time
 import threading
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, GLib
 from gi.repository import GdkPixbuf
 
 import appconsts
-import cairo # Aug-2019 - SvdB - BB
-import editorpersistance # Aug-2019 - SvdB - BB
+import editorpersistance
 import respaths
 import translations
 
@@ -187,21 +186,20 @@ def get_cairo_image(img_name, suffix = ".png", force = None):
     if force == None:
         force = editorpersistance.prefs.double_track_hights
     if force:
-#        new_name = img_name + "@2"
         if img_name[-6:] == "_color":  #editorpersistance.prefs.colorized_icons is True:
              new_name = img_name[:-6] + "@2_color"
         else:
             new_name = img_name + "@2"
     else:
         new_name = img_name
+
     try:
         img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + new_name + suffix)
     except:
         # Colorized icons
-#        img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + suffix)
         if img_name[-6:] == "_color":  #editorpersistance.prefs.colorized_icons is True:
             try:
-                img =  cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + "_color" + suffix)
+                img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + "_color" + suffix)
             except:
                 img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name[:-6] + suffix)
         else:
@@ -209,15 +207,20 @@ def get_cairo_image(img_name, suffix = ".png", force = None):
         # End of Colorized icons
     return img
 
-def get_scaled_cairo_image(img_name):
-    icon = cairo.ImageSurface.create_from_png(img_name)
-    if editorpersistance.prefs.double_track_hights  == False:
+def get_double_scaled_cairo_image(icon_name):
+    img_path = respaths.IMAGE_PATH + icon_name
+    
+    icon = cairo.ImageSurface.create_from_png(img_path)
+    if editorpersistance.prefs.double_track_hights == False:
         return icon
     
+    surface_pattern = cairo.SurfacePattern(icon)
+    surface_pattern.set_filter(cairo.Filter.NEAREST)
+    
     scaled_icon = cairo.ImageSurface(cairo.FORMAT_ARGB32, icon.get_width() * 2, icon.get_height() * 2)
     cr = cairo.Context(scaled_icon)
     cr.scale(2.0, 2.0)
-    cr.set_source_surface(icon, 0, 0)
+    cr.set_source(surface_pattern)
     cr.paint()
 
     return scaled_icon
@@ -270,8 +273,6 @@ def get_slider_row(editable_property, listener, slider_name=None):
     
     editable_property.value_changed_ID = adjustment.connect("value-changed", listener) # saving ID to make it available for disconnect
                                                                                        # This also needs to be after adjustment is set to not loose exiting value for build dummy value 
-    print("editable_property.value_changed_ID")
-
     return (get_two_column_editor_row(name, hbox), hslider)
 
 def get_slider_row_and_spin_widget(editable_property, listener, slider_name=None):
@@ -298,7 +299,6 @@ def get_slider_row_and_spin_widget(editable_property, listener, slider_name=None
     
     editable_property.value_changed_ID = adjustment.connect("value-changed", listener) # saving ID to make it available for disconnect
                                                                                        # This also needs to be available after adjustment is set to not lose exiting value for build dummy value 
-    
     return (get_two_column_editor_row(name, hbox), hslider, spin)
     
 def get_non_property_slider_row(lower, upper, step, value=0, listener=None):
@@ -402,8 +402,8 @@ def pad_label(w, h):
     pad_label.set_size_request(w, h)
     return pad_label
 
-def get_sized_button(lable, w, h, clicked_listener=None):
-    b = Gtk.Button(lable)
+def get_sized_button(button_label, w, h, clicked_listener=None):
+    b = Gtk.Button(label=button_label)
     if clicked_listener != None:
         b.connect("clicked", lambda w,e: clicked_listener())
     b.set_size_request(w, h)
@@ -432,7 +432,7 @@ def get_menu_item(text, callback, data, sensitive=True):
 
 def get_image_menu_item(text, image_name, callback, tooltip_markup=None):
     img = get_image(image_name)
-    text_label = Gtk.Label(text)
+    text_label = Gtk.Label(label=text)
     
     hbox = Gtk.HBox()
     hbox.pack_start(pad_label(4, 4), False, False, 0)
@@ -493,17 +493,59 @@ def remove_children(container):
     for child in children:
         container.remove(child)
 
-class PulseThread(threading.Thread):
+class PulseEvent:
     def __init__(self, proress_bar):
-        threading.Thread.__init__(self)
         self.proress_bar = proress_bar
-
-    def run(self):
         self.exited = False
         self.running = True
-        while self.running:
-            Gdk.threads_enter()
+        
+        Gdk.threads_add_timeout(GLib.PRIORITY_HIGH_IDLE, 100, self._do_pulse)
+                
+    def _do_pulse(self):
+        if self.running:
             self.proress_bar.pulse()
-            Gdk.threads_leave()
-            time.sleep(0.1)
-        self.exited = True
+            return True
+        else:
+            self.exited = True
+            return False
+
+
+def update_text_idle(text_widget, msg):
+    GLib.idle_add(_do_update_text, text_widget, msg)
+
+def _do_update_text(text_widget, msg):
+    text_widget.set_text(msg)
+
+
+class ProgressWindowThread(threading.Thread):
+    def __init__(self, dialog, progress_bar, clip_renderer, callback):
+        self.dialog = dialog
+        self.progress_bar = progress_bar
+        self.clip_renderer = clip_renderer
+        self.callback = callback
+        threading.Thread.__init__(self)
+    
+    def run(self):        
+        self.running = True
+        
+        while self.running:         
+
+            GLib.idle_add(self._update_progress_bar)
+
+            if self.clip_renderer.stopped == True:
+                self.running = False
+                GLib.idle_add(self._render_complete)
+
+            time.sleep(0.33)
+    
+    def _update_progress_bar(self):
+        render_fraction = self.clip_renderer.get_render_fraction()
+        self.progress_bar.set_fraction(render_fraction)
+        pros = int(render_fraction * 100)
+        self.progress_bar.set_text(str(pros) + "%")
+    
+    def _render_complete(self):
+        self.progress_bar.set_fraction(1.0)
+        self.progress_bar.set_text("Render Complete!")
+        self.callback(self.dialog, 0)
+    
\ No newline at end of file
