@@ -23,7 +23,6 @@ from gi.repository import Gtk, GLib, Gdk, GdkPixbuf
 from gi.repository import GObject
 from gi.repository import Pango
 
-import copy
 import os
 import subprocess
 import sys
@@ -31,6 +30,7 @@ import time
 import threading
 
 import appconsts
+import editorlayout
 import editorpersistance
 from editorstate import PROJECT
 import gui
@@ -54,21 +54,18 @@ CONTAINER_CLIP_RENDER_MLT_XML = 2
 CONTAINER_CLIP_RENDER_BLENDER = 3
 MOTION_MEDIA_ITEM_RENDER = 4
 PROXY_RENDER = 5
+CONTAINER_CLIP_RENDER_FLUXITY = 6
 
 open_media_file_callback = None
 
 _status_polling_thread = None
 
 _hamburger_menu = Gtk.Menu()
-
 _jobs_list_view = None
 
 _jobs = [] # proxy objects that represent background renders and provide info on render status.
-
 _remove_list = [] # objects are removed from GUI with delay to give user time to notice copmpletion
 
-jobs_notebook_index = 4 # 4 for single window, app.py sets to 3 for two windows
-
 _jobs_render_progress_window = None
 
 
@@ -105,6 +102,8 @@ class JobProxy: # This object represnts job in job queue.
             return _("Motion Clip")
         elif self.type == PROXY_RENDER:
             return _("Proxy Clip")
+        elif self.type == CONTAINER_CLIP_RENDER_FLUXITY:
+            return _("Generator Clip")
             
     def get_progress_str(self):
         if self.progress < 0.0:
@@ -135,7 +134,7 @@ def add_job(job_proxy):
     _jobs.append(job_proxy)
     _jobs_list_view.fill_data_model()
     if editorpersistance.prefs.open_jobs_panel_on_add == True:
-        gui.middle_notebook.set_current_page(jobs_notebook_index)
+        editorlayout.show_panel(appconsts.PANEL_JOBS)
     
     if editorpersistance.prefs.render_jobs_sequentially == False: # Feature not active for first release 2.6.
         job_proxy.start_render()
@@ -153,7 +152,6 @@ def add_job(job_proxy):
 def update_job_queue(job_msg): # We're using JobProxy objects as messages to update values on jobs in _jobs list.
     global _jobs_list_view, _remove_list
     row = -1
-    job_proxy = None  
     for i in range (0, len(_jobs)):
 
         if _jobs[i].proxy_uid == job_msg.proxy_uid:
@@ -178,7 +176,7 @@ def update_job_queue(job_msg): # We're using JobProxy objects as messages to upd
         _jobs[row].text = _("Completed")
         _jobs[row].progress = 1.0
         _remove_list.append(_jobs[row])
-        GObject.timeout_add(4000, _remove_jobs)
+        GLib.timeout_add(4000, _remove_jobs)
         waiting_jobs = _get_jobs_with_status(QUEUED)
         if len(waiting_jobs) > 0:
             waiting_jobs[0].start_render()
@@ -208,7 +206,7 @@ def _cancel_all_jobs():
 
     _jobs_list_view.fill_data_model()
     _jobs_list_view.scroll.queue_draw()
-    GObject.timeout_add(4000, _remove_jobs)
+    GLib.timeout_add(4000, _remove_jobs)
         
 def get_jobs_of_type(job_type):
     jobs_of_type = []
@@ -234,6 +232,7 @@ def get_jobs_panel():
     global _jobs_list_view
 
     actions_menu = guicomponents.HamburgerPressLaunch(_menu_action_pressed)
+    actions_menu.connect_launched_menu(_hamburger_menu)
     guiutils.set_margins(actions_menu.widget, 8, 2, 2, 18)
 
     row2 =  Gtk.HBox()
@@ -243,8 +242,7 @@ def get_jobs_panel():
     panel = Gtk.VBox()
     panel.pack_start(_jobs_list_view, True, True, 0)
     panel.pack_start(row2, False, True, 0)
-    panel.set_size_request(400, 10)
-
+            
     return panel
 
 
@@ -296,7 +294,7 @@ def _hamburger_item_activated(widget, msg):
 
         _jobs_list_view.fill_data_model()
         _jobs_list_view.scroll.queue_draw()
-        GObject.timeout_add(4000, _remove_jobs)
+        GLib.timeout_add(4000, _remove_jobs)
         
     elif msg == "open_on_add":
         editorpersistance.prefs.open_jobs_panel_on_add = widget.get_active()
@@ -348,7 +346,7 @@ class JobsQueueView(Gtk.VBox):
         self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
-        self.treeview = Gtk.TreeView(self.storemodel)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(True)
         tree_sel = self.treeview.get_selection()
@@ -500,9 +498,10 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
         subprocess.Popen(command_list)
         
     def update_render_status(self):
+        GLib.idle_add(self._update_from_gui_thread)
+            
+    def _update_from_gui_thread(self):
 
-        Gdk.threads_enter()
-                    
         if motionheadless.session_render_complete(self.get_session_id()) == True:
             #remove_as_status_polling_object(self)
             
@@ -524,7 +523,7 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
                     self.progress = 1.0
 
                 self.elapsed = float(elapsed)
-                self.text = _("Rendering Motion Clip ") + self.get_job_name()
+                self.text = self.get_job_name()
                 
                 job_msg = self.get_job_queue_message()
                 
@@ -532,8 +531,6 @@ class MotionRenderJobQueueObject(AbstractJobQueueObject):
             else:
                 # Process start/stop on their own and we hit trying to get non-existing status for e.g completed renders.
                 pass
-
-        Gdk.threads_leave()
     
     def abort_render(self):
         #remove_as_status_polling_object(self)
@@ -575,8 +572,10 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
     
     def update_render_status(self):
 
-        Gdk.threads_enter()
-                    
+        GLib.idle_add(self._update_from_gui_thread)
+            
+    def _update_from_gui_thread(self):
+        
         if proxyheadless.session_render_complete(self.get_session_id()) == True:
             
             job_msg = self.get_completed_job_message()
@@ -596,7 +595,7 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
                     self.progress = 1.0
 
                 self.elapsed = float(elapsed)
-                self.text = _("Rendering Proxy Clip for ") + self.get_job_name()
+                self.text = self.get_job_name()
 
                 job_msg = self.get_job_queue_message()
                 
@@ -604,8 +603,6 @@ class ProxyRenderJobQueueObject(AbstractJobQueueObject):
             else:
                 # Process start/stop on their own and we hit trying to get non-existing status for e.g completed renders.
                 pass
-
-        Gdk.threads_leave()
     
     def abort_render(self):
         # remove_as_status_polling_object(self)
@@ -693,7 +690,9 @@ class JobsRenderProgressWindow:
     def __init__(self, autosave_file):
         self.is_shutting_down = False
         
-        
+        self.save_str = _("Saving...")
+        self.rendering_str = _("Rendering")
+
         # Window
         self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
         self.window.connect("delete-event", lambda w, e:self.close_window())
@@ -723,7 +722,7 @@ class JobsRenderProgressWindow:
 
         self.status_label = Gtk.Label()
         self.status_label.set_text(_("Rendering"))
-        cancel_button = Gtk.Button(_("Cancel All Jobs"))
+        cancel_button = Gtk.Button(label=_("Cancel All Jobs"))
         cancel_button.connect("clicked", lambda w: self.cancel_all())
         
         control_row = Gtk.HBox(False, 0)
@@ -754,31 +753,27 @@ class JobsRenderProgressWindow:
     def jobs_completed(self):
         self.is_shutting_down = True
         
-        Gdk.threads_enter()
-
+        GLib.idle_add(self._jobs_completed_from_gui_thread)
+            
+    def _jobs_completed_from_gui_thread(self):
         self.status_label.set_text(_("Renders Complete."))
-
         self.close_window()
 
-        Gdk.threads_leave()
-
     def update_render_progress(self):
         job = _jobs[0]
         
         if job.status == COMPLETED and self.last_saved_job != id(job) and self.is_shutting_down == False:
-            
-            Gdk.threads_enter()
-            self.status_label.set_text(_("Saving..."))
-            Gdk.threads_leave()
-            
+            GLib.idle_add(self._progress_update_from_gui_thread, self.save_str, job)
+        else:
+            GLib.idle_add(self._progress_update_from_gui_thread, self.rendering_str, job)
+
+    def _progress_update_from_gui_thread(self, status_msg, job):
+        if self.last_saved_job == None:
+            self.status_label.set_text(status_msg)
+        
+        if status_msg == self.save_str:
             self.last_saved_job = id(job)
             self.save_project()
-        else:
-            Gdk.threads_enter()
-            self.status_label.set_text(_("Rendering"))
-            Gdk.threads_leave()
-            
-        Gdk.threads_enter()
 
         elapsed = time.monotonic() - self.start_time
         elapsed_str= "  " + utils.get_time_str_for_sec_float(elapsed)
@@ -788,8 +783,6 @@ class JobsRenderProgressWindow:
         self.render_progress_bar.set_fraction(job.progress)
         self.render_progress_bar.set_text(str(int(job.progress * 100)) + " %")
 
-        Gdk.threads_leave()
-    
     def save_project(self):
         persistance.show_messages = False
         if PROJECT().last_save_path != None:
