@@ -19,17 +19,23 @@
 """
 
 import glob
-import mlt7 as mlt
+import hashlib
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import locale
 import os
 import subprocess
 import sys
 import threading
+from PIL import Image, ImageFont, ImageDraw
+import time
 
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('PangoCairo', '1.0')
-from gi.repository import Gtk, Gdk, GdkPixbuf
+from gi.repository import Gtk, Gdk, GdkPixbuf, GLib, Gio
 from gi.repository import Pango, GObject
 
 import appconsts
@@ -40,10 +46,7 @@ import editorpersistance
 import gui
 import guiutils
 import guicomponents
-import mltenv
-import mltprofiles
-import mlttransitions
-import mltfilters
+import mltinit
 import patternproducer
 import persistance
 import processutils
@@ -55,6 +58,7 @@ import translations
 import userfolders
 import utils
 
+_app = None
 linker_window = None
 target_project = None
 last_media_dir = None
@@ -75,10 +79,6 @@ class ProjectLoadThread(threading.Thread):
         self.filename = filename
 
     def run(self):
-        Gdk.threads_enter()
-        linker_window.project_label.set_text("Loading...")
-        Gdk.threads_leave()
-
         persistance.show_messages = False
         project = persistance.load_project(self.filename, False, True)
         
@@ -87,14 +87,14 @@ class ProjectLoadThread(threading.Thread):
         target_project.c_seq = project.sequences[target_project.c_seq_index]
         _update_media_assets()
 
-        Gdk.threads_enter()
+        GLib.idle_add(self._load_done_update)
+
+    def _load_done_update(self):
         linker_window.relink_list.fill_data_model()
         linker_window.project_label.set_text(self.filename)
         linker_window.set_active_state()
         linker_window.update_files_info()
         linker_window.load_button.set_sensitive(False)
-        Gdk.threads_leave()
-
 
 class MediaLinkerWindow(Gtk.Window):
     def __init__(self):
@@ -104,7 +104,7 @@ class MediaLinkerWindow(Gtk.Window):
         app_icon = GdkPixbuf.Pixbuf.new_from_file(respaths.IMAGE_PATH + "flowblademedialinker.png")
         self.set_icon(app_icon)
 
-        load_button = Gtk.Button(_("Load Project For Relinking"))
+        load_button = Gtk.Button(label=_("Load Project For Relinking"))
         load_button.connect("clicked",
                             lambda w: self.load_button_clicked())
         self.load_button = load_button
@@ -134,9 +134,11 @@ class MediaLinkerWindow(Gtk.Window):
         
         self.relink_list = MediaRelinkListView()
 
-        self.find_button = Gtk.Button(_("Set File Relink Path"))
+        self.find_button = Gtk.Button(label=_("Set File Relink Path"))
         self.find_button.connect("clicked", lambda w: _set_button_pressed())
-        self.delete_button = Gtk.Button(_("Delete File Relink Path"))
+        self.create_button = Gtk.Button(label=_("Create Placeholder File"))
+        self.create_button.connect("clicked", lambda w: _create_relink_media_button_pressed())
+        self.delete_button = Gtk.Button(label=_("Delete File Relink Path"))
         self.delete_button.connect("clicked", lambda w: _delete_button_pressed())
 
         self.display_combo = Gtk.ComboBoxText()
@@ -148,20 +150,23 @@ class MediaLinkerWindow(Gtk.Window):
         buttons_row = Gtk.HBox(False, 2)
         buttons_row.pack_start(self.display_combo, False, False, 0)
         buttons_row.pack_start(Gtk.Label(), True, True, 0)
+        buttons_row.pack_start(self.create_button, False, False, 0)
+        buttons_row.pack_start(guiutils.pad_label(24, 4), False, False, 0)
         buttons_row.pack_start(self.delete_button, False, False, 0)
-        buttons_row.pack_start(guiutils.pad_label(4, 4), False, False, 0)
+        buttons_row.pack_start(guiutils.pad_label(24, 4), False, False, 0)
         buttons_row.pack_start(self.find_button, False, False, 0)
 
-        self.save_button = Gtk.Button(_("Save Relinked Project As..."))
+        self.save_button = Gtk.Button(label=_("Save Relinked Project As..."))
         self.save_button.connect("clicked", lambda w:_save_project_pressed())
-        cancel_button = Gtk.Button(_("Close"))
+        cancel_button = Gtk.Button(label=_("Close"))
         cancel_button.connect("clicked", lambda w:_shutdown())
         dialog_buttons_box = Gtk.HBox(True, 2)
         dialog_buttons_box.pack_start(cancel_button, True, True, 0)
         dialog_buttons_box.pack_start(self.save_button, False, False, 0)
         
+        self.msg_label = Gtk.Label(label="")
         dialog_buttons_row = Gtk.HBox(False, 2)
-        dialog_buttons_row.pack_start(Gtk.Label(), True, True, 0)
+        dialog_buttons_row.pack_start(self.msg_label, True, True, 0)
         dialog_buttons_row.pack_start(dialog_buttons_box, False, False, 0)
 
         pane = Gtk.VBox(False, 2)
@@ -198,6 +203,8 @@ class MediaLinkerWindow(Gtk.Window):
             dialog.destroy()
 
     def load_project(self, filename):
+        linker_window.project_label.set_text(_("Loading..."))
+        
         global load_thread
         load_thread = ProjectLoadThread(filename)
         load_thread.start()
@@ -223,6 +230,7 @@ class MediaLinkerWindow(Gtk.Window):
         self.found_count.set_sensitive(active) 
         self.project_label.set_sensitive(active) 
         self.proj.set_sensitive(active) 
+        self.create_button.set_sensitive(active)
 
     def update_files_info(self):
         found = 0
@@ -262,11 +270,10 @@ class MediaRelinkListView(Gtk.VBox):
         self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
-        self.treeview = Gtk.TreeView(self.storemodel)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(True)
         self.treeview.connect("button-press-event", self.row_pressed)
-        tree_sel = self.treeview.get_selection()
 
         # Column views
         self.missing_text = _("Missing Media File Path")
@@ -353,9 +360,12 @@ class MediaRelinkListView(Gtk.VBox):
 # ----------------------------------------------------------- logic
 class MediaAsset:
 
-    def __init__(self, orig_path, media_type):
+    # Default file for missing files to relink
+    def __init__(self, orig_path, media_type, media_length=0):
         self.orig_path = orig_path
         self.media_type = media_type
+        self.length = media_length
+    # End of Default file for missing files to relink
 
         self.orig_file_exists = os.path.isfile(orig_path)
         if self.media_type == appconsts.IMAGE_SEQUENCE:
@@ -385,7 +395,9 @@ def _update_media_assets():
         if isinstance(media_file, patternproducer.AbstractBinClip):
             continue
         try:
-            new_assets.append(MediaAsset(media_file.path, media_file.type))
+            # Default file for missing files to relink
+            new_assets.append(MediaAsset(media_file.path, media_file.type, media_file.length))
+            # End of Default file for missing files to relink
             asset_paths[media_file.path] = media_file.path
         except:
             print("failed loading:", media_file)
@@ -398,7 +410,9 @@ def _update_media_assets():
                 # Only producer clips are affected
                 if (clip.is_blanck_clip == False and (clip.media_type != appconsts.PATTERN_PRODUCER)):
                     if not(clip.path in asset_paths):
-                        new_assets.append(MediaAsset(clip.path, clip.media_type))
+                        # Default file for missing files to relink
+                        new_assets.append(MediaAsset(clip.path, clip.media_type,clip.clip_out - clip.clip_in + 1))  #clip.get_length()))
+                        # End of Default file for missing files to relink
                         asset_paths[clip.path] = clip.path
         # Wipe lumas
         for compositor in seq.compositors:
@@ -426,6 +440,8 @@ def _media_asset_menu_item_selected(widget, data):
         _delete_relink_path(media_asset)
     if msg == "show path":
         _show_paths(media_asset)
+    if msg == "create placeholder":
+        _create_relink_media_button_pressed()
 
 def _set_button_pressed():
     media_asset = linker_window.get_selected_media_asset()
@@ -435,10 +451,77 @@ def _set_button_pressed():
 
 def _set_relink_path(media_asset):
     file_name = os.path.basename(media_asset.orig_path)
+    # End of Default file for missing files to relink
     dialogs.media_file_dialog(_("Select Media File To Relink To") + " " + file_name, 
                                 _select_relink_path_dialog_callback, False, 
                                 media_asset, linker_window, last_media_dir)
 
+def _create_relink_media_button_pressed():
+    media_asset = linker_window.get_selected_media_asset()
+
+    if media_asset.media_type == appconsts.IMAGE:
+        info_text = _("Creating placeholder image...")
+    elif media_asset.media_type == appconsts.AUDIO:
+        info_text = _("Creating placeholder audio...")
+    elif media_asset.media_type == appconsts.VIDEO:
+        info_text = _("Creating placeholder video...")
+
+    linker_window.msg_label.set_text(info_text)
+
+    # We need to launch render on another thread and return from this function
+    # as soon as possible or we will freeze GUI because we hold GDK lock during render.
+    render_thread = MediaRecreateThread(media_asset) 
+    render_thread.start()
+
+    # Poll rendering from GDK with timeout events to get access to GDK lock on updates 
+    # to be able to draw immediately.
+    Gdk.threads_add_timeout(GLib.PRIORITY_HIGH_IDLE, 500, _recreate_update, render_thread, info_text)
+
+def _recreate_update(render_thread, info_text):
+    elapsed = utils.get_time_str_for_sec_float(render_thread.get_elapsed())
+    linker_window.msg_label.set_text(info_text + " " + elapsed)
+    linker_window.msg_label.queue_draw()
+
+    if render_thread.running == False:
+        linker_window.relink_list.fill_data_model()
+        linker_window.msg_label.set_text(_("Place holder media ready."))
+        return False # This stops repeated calls to this function.
+    else:
+        return True # Function will be called again in 500ms.
+            
+class MediaRecreateThread(threading.Thread):
+    def __init__(self, media_asset):
+        threading.Thread.__init__(self)
+        self.media_asset = media_asset
+        self.file_name = os.path.basename(media_asset.orig_path)
+        self.running = True
+        
+    def run(self):
+        
+        self.start_time = time.monotonic()
+
+        new_media_file_name = hashlib.md5(str(os.urandom(32)).encode('utf-8') +  str.encode(self.file_name)).hexdigest()
+        link_path_no_ext = userfolders.get_render_dir() + "/" + new_media_file_name
+        
+        if self.media_asset.media_type == appconsts.IMAGE:
+            link_path = link_path_no_ext + ".png"
+            self.success = background_missing_image(link_path, self.file_name)
+        elif self.media_asset.media_type == appconsts.AUDIO:
+            link_path = link_path_no_ext + ".wav"
+            self.success = sine_wav(link_path, self.media_asset.length)
+        elif self.media_asset.media_type == appconsts.VIDEO:
+            link_path = link_path_no_ext + ".mp4"
+            self.success = video_file_replace(link_path, self.file_name, self.media_asset.length)
+
+        self.media_asset.relink_path = link_path
+        
+        self.running = False
+    
+        # TODO: handle self.success == False with something
+    
+    def get_elapsed(self):
+        return time.monotonic() - self.start_time 
+        
 def _select_relink_path_dialog_callback(file_select, response_id, media_asset):
     filenames = file_select.get_filenames()
     file_select.destroy()
@@ -454,17 +537,17 @@ def _select_relink_path_dialog_callback(file_select, response_id, media_asset):
     global last_media_dir
     last_media_dir = folder
 
+    # Default file for missing files to relink
+    linker_window.set_title( folder + " " + file_name )    
+    #End of Default file for missing files to relink
+
     # Relink all the files in a same directory
     for med_asset in media_assets:
         med_link_name = os.path.basename(med_asset.orig_path)
         link_path = os.path.join(folder, med_link_name)
         if os.path.isfile(link_path):
             if med_asset.media_type == appconsts.IMAGE_SEQUENCE: # img seqs need formatted path
-                if editorstate.mlt_version_is_equal_or_greater("0.8.5"):
-                    new_style = True
-                else:
-                    new_style = False
-                resource_name_str = utils.get_img_seq_resource_name(link_path, new_style)
+                resource_name_str = utils.get_img_seq_resource_name(link_path)
                 med_asset.relink_path = folder + "/" + resource_name_str
             else:
                 med_asset.relink_path = link_path
@@ -572,6 +655,70 @@ def _relink_project_media_paths():
                 if res_path in relinked_paths:
                     propertyparse.set_property_value(compositor.transition.properties,  "composite.luma", relinked_paths[res_path])
 
+# Default file for missing files to relink
+
+def background_missing_image(link_path, file_name, video_file_name=None):
+    screen_width = 1920 #target_project.profile.width()
+    screen_height  = 1080 #target_project.profile.height()
+
+    if os.path.isfile(link_path):
+        os.remove(link_path)
+
+    size = (screen_width, screen_height)
+    font = ImageFont.truetype("/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf", 40)
+    img = Image.new("RGBA",size, (200,40, 40))
+    draw = ImageDraw.Draw(img)
+    if video_file_name != None: # We use this to make images for both grphics and video media.
+        new_file_name = video_file_name
+    else:
+        new_file_name = link_path
+    draw.text((30,20), " >>>  " + new_file_name + "\n\nreplaces the missing file:\n\n<<<  " + file_name, (255,255,0), font=font)
+    draw = ImageDraw.Draw(img)
+    img.save(link_path)
+    
+    if os.path.isfile(link_path):
+        return True
+    else:
+        return False
+
+def sine_wav(link_path, duration):
+    if os.path.isfile(link_path):
+        os.remove(link_path)
+
+    duration = int(duration/25) + 1
+    dur = '\"sine=frequency=1000:duration="' + str(duration) + '"\" '
+    command = 'ffmpeg -f lavfi -i ' +  dur + ' ' + link_path
+
+    os.system(command)
+    
+    if os.path.isfile(link_path):
+        return True
+    else:
+        return False
+
+def  video_file_replace(link_path, file_name, duration):
+
+    image_file = "/tmp/tmp.png"
+    if os.path.isfile(image_file):
+        os.remove(image_file)
+    success = background_missing_image(image_file, file_name)
+    
+    audio_file = "/tmp/tmp.wav"
+    if os.path.isfile(audio_file):
+        os.remove(audio_file)
+    success = sine_wav(audio_file, duration)
+    
+    if os.path.isfile(link_path):
+        os.remove(link_path)
+    command = "ffmpeg -loop 1 -i "+ image_file + " -i " +  audio_file  + " -c:v libx264 -c:a aac -strict experimental -b:a 192k -shortest " + link_path
+    print("ml 696",  command)
+    os.system(command)
+
+    if os.path.isfile(link_path):
+        return True
+    else:
+        return False
+
 
 # ----------------------------------------------------------- main
 def main(root_path, filename):
@@ -591,51 +738,36 @@ def main(root_path, filename):
 
     # Load editor prefs and list of recent projects
     editorpersistance.load()
-    
-    # Init translations module with translations data
-    translations.init_languages()
-    translations.load_filters_translations()
-    mlttransitions.init_module()
 
-    # Init gtk threads
-    Gdk.threads_init()
-    Gdk.threads_enter()
+    # Create app.
+    global _app
+    _app = MediaLinkerApp()
+    _app.filename = filename
+    _app.run(None)
 
-    # Themes
-    if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
-        Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME \
-            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY \
-            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
-            gui.apply_gtk_css(editorpersistance.prefs.theme)
 
-    repo = mlt.Factory().init()
-    processutils.prepare_mlt_repo(repo)
-    
-    # Set numeric locale to use "." as radix, MLT initilizes this to OS locale and this causes bugs 
-    locale.setlocale(locale.LC_NUMERIC, 'C')
+class MediaLinkerApp(Gtk.Application):
+    def __init__(self, *args, **kwargs):
+        Gtk.Application.__init__(self, application_id=None,
+                                 flags=Gio.ApplicationFlags.FLAGS_NONE)
+        self.connect("activate", self.on_activate)
 
-    # Check for codecs and formats on the system
-    mltenv.check_available_features(repo)
-    renderconsumer.load_render_profiles()
+    def on_activate(self, data=None):
+        # Themes
+        gui.apply_theme(editorpersistance.prefs.theme)
 
-    # Load filter and compositor descriptions from xml files.
-    mltfilters.load_filters_xml(mltenv.services)
-    mlttransitions.load_compositors_xml(mltenv.transitions)
+        # Init mlt.
+        repo = mltinit.init_with_translations()
+        
+        appconsts.SAVEFILE_VERSION = projectdata.SAVEFILE_VERSION
 
-    # Create list of available mlt profiles
-    mltprofiles.load_profile_list()
+        global linker_window
+        linker_window = MediaLinkerWindow()
 
-    appconsts.SAVEFILE_VERSION = projectdata.SAVEFILE_VERSION
+        if self.filename != NO_PROJECT_AT_LAUNCH:
+            linker_window.load_project(self.filename)
 
-    global linker_window
-    linker_window = MediaLinkerWindow()
-
-    if filename != NO_PROJECT_AT_LAUNCH:
-        linker_window.load_project(filename)
-
-    Gtk.main()
-    Gdk.threads_leave()
-    
+        self.add_window(linker_window)
+        
 def _shutdown():
-    Gtk.main_quit()
+    _app.quit()
