@@ -24,7 +24,11 @@ a Xwindow of a GTK+ widget and os audiosystem using a SDL consumer.
 """
 from gi.repository import Gdk
 
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
+    
 import os
 import time
 
@@ -178,7 +182,7 @@ class Player:
     def start_playback(self):
         """
         Starts playback from current producer
-        """
+        """        
         self.producer.set_speed(1)
         self.ticker.stop_ticker()
         self.ticker.start_ticker()
@@ -196,6 +200,10 @@ class Player:
         """
         Stops playback from current producer
         """
+        self.loop_start = -1 # User possibly goes into marks looping but stops without Control key.
+        self.loop_end = -1
+        self.is_looping = False
+
         self.ticker.stop_ticker()
         self.producer.set_speed(0)
         updater.update_frame_displayers(self.producer.frame())
@@ -213,6 +221,22 @@ class Player:
         self.ticker.stop_ticker()
         self.ticker.start_ticker()
 
+    def start_loop_playback_range(self, range_in, range_out):
+        seq_len = self.producer.get_length()
+        if range_in >= seq_len:
+            return
+        if range_out > seq_len:
+            range_out = seq_len
+        
+        self.loop_start = range_in
+        self.loop_end = range_out
+        
+        self.is_looping = True
+        self.seek_frame(self.loop_start, False)
+        self.producer.set_speed(1)
+        self.ticker.stop_ticker()
+        self.ticker.start_ticker()
+
     def stop_loop_playback(self, looping_stopped_callback):
         """
         Stops playback from current producer
@@ -281,7 +305,7 @@ class Player:
         frame.set("consumer_deinterlace", 1)
 
         # Now we are ready to get the image and save it.        
-        rgb = frame.get_image(int(mlt.mlt_image_rgb24a), int(self.profile.width()), int(self.profile.height()))
+        rgb = frame.get_image(int(mlt.mlt_image_rgba), int(self.profile.width()), int(self.profile.height()))
         return rgb
 
     def display_inside_sequence_length(self, new_seq_len):
@@ -315,15 +339,10 @@ class Player:
 
         # If we're currently rendering, set progress bar and exit event handler.
         if self.is_rendering:
-            if (self.producer.get_length() - 1) < 1:
-                render_fraction = 1.0
-            else:
-                render_fraction = ((float(current_frame - self.render_start_frame)) / 
-                  (float(self.render_stop_frame - self.render_start_frame)))
             self.render_gui_update_count = self.render_gui_update_count + 1
             if self.render_gui_update_count % 8 == 0: # we need quick updates for stop accuracy, but slower gui updating
                 self.render_gui_update_count = 1
-            return 
+            return
 
         # If we're out of active range seek end.
         if current_frame >= self.get_active_length():
