@@ -21,7 +21,10 @@ from gi.repository import Gdk
 from gi.repository import Gtk, GLib
 
 import cairo
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import numpy as np
 import os
 import threading
@@ -541,11 +544,12 @@ class MonitorWidget:
     # ------------------------------------------------------------------ MATCH FRAME
     def match_frame_write_complete(self, frame_name):
         self.match_frame_surface = self.create_match_frame_image_surface(frame_name)
-        
-        Gdk.threads_enter()
+        GLib.timeout_add(0, self._draw_displays)
+            
+    def _draw_displays(self):
+        # This is called from GUI prosess, we can do Gtk calls.
         self.left_display.queue_draw()
         self.right_display.queue_draw()
-        Gdk.threads_leave()
 
     def create_pattern_producer_match_frame(self):        
         w, h = self.get_match_frame_panel_size()
@@ -582,7 +586,6 @@ class MonitorWidget:
         surface = cairo.ImageSurface.create_from_png(matchframe_path)
 
         # Create and return scaled surface
-        profile_screen_ratio = float(PROJECT().profile.width()) / float(PROJECT().profile.height())
         match_frame_width, match_frame_height = self.get_match_frame_panel_size()
         
         scaled_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(match_frame_width), int(match_frame_height))
@@ -596,7 +599,7 @@ class MonitorWidget:
     
     def _get_cairo_buf_from_mlt_rgb(self, screen_rgb_data, img_w, img_h ):
         buf = np.frombuffer(screen_rgb_data, dtype=np.uint8)
-        buf.shape = (img_h + 1, img_w, 4) # +1 in h, seemeed to need it
+        buf.shape = (img_h, img_w, 4)
         out = np.copy(buf)
         r = np.index_exp[:, :, 0]
         b = np.index_exp[:, :, 2]
@@ -708,8 +711,6 @@ class MonitorWidget:
         # Draw active screen indicator and compute tc and frame delta positions
         cr.set_source_rgb(*MONITOR_INDICATOR_COLOR)
 
-        match_tc_x = 0
-        edit_tc_x = 0
         delta_frames_x = 0
         if self.view == START_TRIM_VIEW:
             cr.rectangle(w/2, 0, w/2, 4)
@@ -740,9 +741,7 @@ class MonitorWidget:
 
         # Draw active screen indicator and compute tc and frame delta positions
         cr.set_source_rgb(*MONITOR_INDICATOR_COLOR)
-       
-        match_tc_x = 0
-        edit_tc_x = 0
+
         delta_frames_x = 0
         if self.view == ROLL_TRIM_RIGHT_ACTIVE_VIEW:
             cr.rectangle(w/2, 0, w/2, 4)
@@ -781,8 +780,6 @@ class MonitorWidget:
         # Draw active screen indicator and compute tc and frame delta positions
         cr.set_source_rgb(*MONITOR_INDICATOR_COLOR)
        
-        match_tc_x = 0
-        edit_tc_x = 0
         delta_frames_x = 0
         if self.view == SLIP_TRIM_RIGHT_ACTIVE_VIEW:
             cr.rectangle(w/2, 0, w/2, 4)
@@ -918,6 +915,7 @@ class MatchSurfaceCreator(threading.Thread):
         threading.Thread.__init__(self)
         
     def run(self):
+        print("MatchSurfaceCreator.run")
         # Create new producer to get mlt frame data
         while _producer == None:
             print("MatchSurfaceCreator: waiting for _producer")
@@ -932,7 +930,7 @@ class MatchSurfaceCreator(threading.Thread):
         # And make sureto deinterlace if input is interlaced
         frame.set("consumer_deinterlace", 1)
         size = _widget.get_match_frame_panel_size()
-        mlt_rgb = frame.get_image(mlt.mlt_image_rgb24a, *size) 
+        mlt_rgb = frame.get_image(mlt.mlt_image_rgba, *size) 
    
         # Create cairo surface
         cairo_buf = _widget._get_cairo_buf_from_mlt_rgb(mlt_rgb, *size)
@@ -943,10 +941,7 @@ class MatchSurfaceCreator(threading.Thread):
         _widget.match_frame_surface = surface
         
         # Repaint
-        Gdk.threads_enter()
-        _widget.left_display.queue_draw()
-        _widget.right_display.queue_draw()
-        Gdk.threads_leave()
-        
+        GLib.timeout_add(0, _widget._draw_displays)
+
         
         
