@@ -27,7 +27,10 @@ import hashlib
 
 from gi.repository import Gtk, Gdk
 
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 
 import appconsts
 import dialogutils
@@ -38,6 +41,7 @@ import mltrefhold
 import respaths
 import userfolders
 import utils
+import updater
 
 # Pattern producer types
 UNDEFINED = 0
@@ -89,27 +93,7 @@ def _create_ising_clip_callback(dialog, response_id, widgets):
         _update_gui_for_pattern_producer_media_object_add()
         
     dialog.destroy()
-"""
-def create_color_pulse_clip():
-    _color_pulse_clip_dialog(_create_color_pulse_clip_callback)
 
-def _create_color_pulse_clip_callback(dialog, response_id, widgets):
-    if response_id == Gtk.ResponseType.ACCEPT:
-        media_object = BinColorPulseClip(PROJECT().next_media_file_id, _("Color Pulse"))
-
-        s1_slider, s2_slider, s3_slider, s4_slider, m1_slider, m2_slider = widgets
-        media_object.set_property_values(s1_slider.get_adjustment().get_value() / 100.0,
-                                         s2_slider.get_adjustment().get_value() / 100.0, 
-                                         s3_slider.get_adjustment().get_value() / 100.0,
-                                         s4_slider.get_adjustment().get_value() / 100.0,
-                                         m1_slider.get_adjustment().get_value() / 100.0,
-                                         m2_slider.get_adjustment().get_value() / 100.0)
-
-        PROJECT().add_pattern_producer_media_object(media_object)
-        _update_gui_for_pattern_producer_media_object_add()
-
-    dialog.destroy()
-"""
 def create_count_clip():
     media_object = BinCountClip(PROJECT().next_media_file_id, _("Count"))
     PROJECT().add_pattern_producer_media_object(media_object)
@@ -117,7 +101,7 @@ def create_count_clip():
 
 def _update_gui_for_pattern_producer_media_object_add():
     gui.media_list_view.fill_data_model()
-    gui.bin_list_view.fill_data_model()
+    updater.update_current_bin_files_count()
 
 # ---------------------------------------------------- 
 def create_pattern_producer(profile, bin_clip):
@@ -140,7 +124,7 @@ def create_pattern_producer(profile, bin_clip):
     return clip
 
 # --------------------------------------------------- DECPRECATED producer create methods
-# --------------------------------------------------- REMOVE 2017
+# --------------------------------------------------- LOOK TO REMOVE.
 """
 We originally did producer creation using elifs and now using pickle() for save/load 
 requires keeping this around until atleast 2017 for backwards compatibility.
@@ -328,14 +312,6 @@ class BinCountClip(AbstractBinClip):
 
     def set_property_values(self):
         pass
-        """
-        self.s1 = s1
-        self.s2 = s2
-        self.s3 = s3
-        self.s4 = s4
-        self.m1 = m1
-        self.m2 = m2
-        """
         
     def create_mlt_producer(self, profile):
         producer = mlt.Producer(profile, "count")    
@@ -375,7 +351,7 @@ def _color_clip_dialog(callback):
     vbox.pack_start(row2, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
     
-    align = guiutils.set_margins(vbox, 12, 0, 12, 12)
+    guiutils.set_margins(vbox, 12, 0, 12, 12)
 
     selection_widgets = (name_entry, color_button)
 
@@ -455,4 +431,3 @@ def _color_pulse_clip_dialog(callback):
     dialogutils.default_behaviour(dialog)
     dialogutils.set_outer_margins(dialog.vbox)
     dialog.show_all()
-    
