@@ -32,9 +32,10 @@ import fnmatch
 import hashlib
 import os
 import pickle
+import sys
 import time
 
-from gi.repository import Gdk
+from gi.repository import GLib
 
 import appconsts
 import atomicfile
@@ -43,7 +44,6 @@ import editorpersistance
 import mltprofiles
 import mltfilters
 import mlttransitions
-import miscdataobjects
 import persistancecompat
 import propertyparse
 import resync
@@ -62,8 +62,8 @@ MEDIA_FILE_REMOVE = ['icon']
 
 # Used to flag a not found relative path
 NOT_FOUND = "/not_found_not_found/not_found"
-
-# Used to send messages when loading project
+        
+# Used to send messages when loading project, set at callsite.
 load_dialog = None
 
 # These are used to recrete parenting relationships
@@ -90,6 +90,10 @@ _fps_conv_mult = 1.0
 # A dict is put here when saving for profile change to contain paths to changed MLT XML files
 _xml_new_paths_for_profile_change = None
 
+# MLT removed quite few services for 7.0 and we need to inform users if any of those cannot be loaded.
+dead_compositors = 0
+
+
 class FileProducerNotFoundError(Exception):
 
     def __init__(self, value):
@@ -105,19 +109,19 @@ class ProjectProfileNotFoundError(Exception):
         return repr(self.value)
 
 # -------------------------------------------------- LOAD MESSAGES
-def _show_msg(msg, delay=0.0):
+def _show_msg(msg):
     if show_messages == True:
-        Gdk.threads_enter()
-        load_dialog.info.set_text(msg)
-        time.sleep(delay)
-        Gdk.threads_leave()
+        GLib.idle_add(_do_show_msg, msg)
 
+def _do_show_msg(msg):
+    load_dialog.info.set_text(msg)
+        
 # -------------------------------------------------- SAVE
 def save_project(project, file_path, changed_profile_desc=None):
     """
     Creates pickleable project object
     """
-    print("Saving project...")  # + os.path.basename(file_path)
+    print("Saving project...")# + os.path.basename(file_path))
     
     # Get shallow copy
     s_proj = copy.copy(project)
@@ -256,7 +260,7 @@ def get_p_clip(clip):
             pass 
 
     # Set 'type' attribute for MLT object type
-    # This IS NOT USED anywhere anymore and should be removed.
+    # This is NOT USED anywhere anymore and should be removed.
     s_clip.type = 'Mlt__Producer'
 
     # Get replace filters
@@ -385,7 +389,7 @@ def _save_changed_xml_file(s_media_file, new_profile):
 def load_project(file_path, icons_and_thumnails=True, relinker_load=False):
     _show_msg("Unpickling")
 
-    project = utils.unpickle(file_path)
+    project = unpickle(file_path)
 
     # Relinker only operates on pickleable python data 
     if relinker_load:
@@ -414,7 +418,7 @@ def load_project(file_path, icons_and_thumnails=True, relinker_load=False):
         raise ProjectProfileNotFoundError(project.profile_desc)
 
     for k, media_file in project.media_files.items():
-        media_file.current_frame = 0 # this is always reset on load, value is not considered persistent
+        media_file.current_frame = 0 # this is always reset on load, value is not considered persistent.
 
         # Avoid crash in case path attribute is missing (color clips).
         # All code in loop below handles issues not related to color clips.
@@ -453,6 +457,8 @@ def load_project(file_path, icons_and_thumnails=True, relinker_load=False):
             if os.path.isfile(media_file.second_file_path): # Original media file exists, use it
                 media_file.set_as_original_media_file()
 
+        _show_msg("Loading Media Item: " + media_file.name)
+
     # Add MLT objects to sequences.
     global all_clips, sync_clips
     seq_count = 1
@@ -495,10 +501,8 @@ def fill_sequence_mlt(seq, SAVEFILE_VERSION):
     # Create tractor, field, multitrack
     seq.init_mlt_objects()
 
-    # Compositing mode COMPOSITING_MODE_TOP_DOWN_AUTO_FOLLOW was removed 2.6->,  we just convert it 
-    # to COMPOSITING_MODE_TOP_DOWN_FREE_MOVE and compositors now work
-    if seq.compositing_mode == appconsts.COMPOSITING_MODE_TOP_DOWN_AUTO_FOLLOW:
-        seq.compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
+    # Compositing mode COMPOSITING_MODE_TOP_DOWN_AUTO_FOLLOW was removed 2.6->
+    persistancecompat.FIX_DEPRECATED_SEQUENCE_COMPOSITING_MODE(seq)
     
     # Grap and replace py tracks. Do this way to use same create
     # method as when originally created.
@@ -511,6 +515,8 @@ def fill_sequence_mlt(seq, SAVEFILE_VERSION):
     # Create and fill MLT tracks.
     for py_track in py_tracks:
         mlt_track = seq.add_track(py_track.type)
+        if py_track.id == len(py_tracks) - 1:
+            continue # Do not try to fill hidden track even if somehow a clip was saved there.
         fill_track_mlt(mlt_track, py_track)
         # Set audio gain and pan filter values
         if hasattr(mlt_track, "gain_filter"): # Hidden track and black track do not have these
@@ -520,12 +526,21 @@ def fill_sequence_mlt(seq, SAVEFILE_VERSION):
     
     # Create and connect compositors.
     mlt_compositors = []
+    global dead_compositors
+    dead_compositors = 0
+                
     for py_compositor in seq.compositors:
             # Keeping backwards compability
             persistancecompat.FIX_MISSING_COMPOSITOR_ATTRS(py_compositor)
-                
+
             # Create new compositor object
-            compositor = mlttransitions.create_compositor(py_compositor.type_id)                                        
+            compositor = mlttransitions.create_compositor(py_compositor.type_id)
+
+            # Some Compositors have been removed from MLT, and cannot be created on load.
+            if compositor == None:
+                dead_compositors += 1
+                continue
+
             compositor.create_mlt_objects(seq.profile)
 
             # Copy and set param values
@@ -546,6 +561,9 @@ def fill_sequence_mlt(seq, SAVEFILE_VERSION):
     seq.compositors = mlt_compositors
     seq.restack_compositors()
 
+    # Apply the STANDARD FULL TRACK COMPOSITORR change done for 2.10. 
+    persistancecompat.FIX_FULLTRACK_COMPOSITING_MODE_COMPOSITORS(seq)
+    
     # Connect sync relations
     for clip_n_track in sync_clips:
         clip, track = clip_n_track
@@ -576,6 +594,9 @@ def fill_track_mlt(mlt_track, py_track):
     sequence = mlt_track.sequence
     for i in range(0, len(py_track.clips)):
         clip = py_track.clips[i]
+        if clip.is_blanck_clip == False:
+            _show_msg(_("Building track ") + str(py_track.id) + " - " + clip.name)
+                
         mlt_clip = None
         append_created = True # blanks get appended at creation time, other clips don't
 
@@ -594,6 +615,7 @@ def fill_track_mlt(mlt_track, py_track):
 
             # Try to fix possible missing proxy files for clips if we are in proxy mode.
             if not os.path.isfile(clip.path) and project_proxy_mode == appconsts.USE_PROXY_MEDIA:
+                
                 try:
                     try:
                         possible_orig_file_path = proxy_path_dict[clip.path] # This dict was filled with media file data.
@@ -611,10 +633,17 @@ def fill_track_mlt(mlt_track, py_track):
 
             # If container clip rendered media is missing try to use unrendered media.
             if not os.path.isfile(clip.path) and clip.container_data != None:
-                if clip.path != clip.container_data.unrendered_media:
-                    clip.path = clip.container_data.unrendered_media
-                    clip.container_data.clear_rendered_media()
-                    
+                if clip.media_type != appconsts.IMAGE_SEQUENCE:
+                    if clip.path != clip.container_data.unrendered_media:
+                        clip.path = clip.container_data.unrendered_media
+                        clip.container_data.clear_rendered_media()
+                else:
+                    folder = os.path.dirname(clip.path)
+                    if not(os.path.isdir(folder)) or len(os.listdir(os.path.dirname(clip.path))) == 0:
+                        if clip.path != clip.container_data.unrendered_media:
+                            clip.path = clip.container_data.unrendered_media
+                            clip.container_data.clear_rendered_media()
+
             mlt_clip = sequence.create_file_producer_clip(clip.path, None, False, clip.ttl)
             
             if mlt_clip == None:
@@ -745,7 +774,7 @@ def get_img_seq_media_path(path, load_file_path):
 
 def get_relative_path(project_file_path, asset_path):
     name = os.path.basename(asset_path)
-    _show_msg(_("Relative file search for ")  + name + "...", delay=0.0)
+    _show_msg(_("Relative file search for ")  + name + "...")
     matches = []
     asset_folder, asset_file_name = os.path.split(asset_path)
     project_folder, project_file_name =  os.path.split(project_file_path)
@@ -762,8 +791,7 @@ def get_relative_path(project_file_path, asset_path):
 
 def get_img_seq_relative_path(project_file_path, asset_path):
     name = os.path.basename(asset_path)
-    _show_msg(_("Relative file search for ")  + name + "...", delay=0.0)
-    matches = []
+    _show_msg(_("Relative file search for ")  + name + "...")
     asset_folder, asset_file_name = os.path.split(asset_path)
     look_up_file_name = utils.get_img_seq_glob_lookup_name(asset_file_name)
     
@@ -796,3 +824,32 @@ compositors_index_to_type_id = ["##affine","##opacity_kf","##pict_in_pict", "##r
                                 "##grain_extract", "##grain_merge", "##hardlight", "##hue", "##lighten",
                                 "##multiply", "##overlay", "##saturation", "##screen", "##softlight",
                                 "##subtract", "##value"]
+
+# ------------------------------------------------------- unpickling with mlt module fixes
+def unpickle(path):
+    try:
+        f = open(path, "rb")
+        return pickle.load(f)
+    except:
+        # Fix for opening projects when mlt available as 'mlt7' but saved with 
+        # mlt available as 'mlt'.
+        #
+        # Because sequence.py does 'import mlt' (and now does 'import mlt7') we still 
+        # import it on pickle.load() even if no MLT objects are pickled.
+        # Once mlt Python bindigs name changed from 'mlt to 'mlt7' the unpickle failed.
+        # We need to do 'sys.modules["mlt"] = mlt' after 'import mlt7 as mlt'
+        # to make module 'mlt' available when doing unpickling.
+        #
+        # NOTE: It may not be possible to remove mlt import from pickle data, but we need to 
+        # make an effort.
+
+        print("Fixing for unpickling doing 'import mlt'...")
+        
+        import mlt7 as mlt
+        sys.modules["mlt"] = mlt
+        try:
+            f = open(path, "rb")
+            return pickle.load(f)
+        except:
+            f = open(path, "rb")
+            return pickle.load(f, encoding='latin1') 
