@@ -23,7 +23,6 @@ import os
 from gi.repository import Gtk
 
 import appconsts
-import dialogs
 import dialogutils
 import editorpersistance
 import gui
@@ -31,6 +30,7 @@ import guiutils
 import mltprofiles
 import multiprocessing
 import utils
+import utilsgtk
 
 PREFERENCES_WIDTH = 730
 PREFERENCES_HEIGHT = 440
@@ -38,7 +38,6 @@ PREFERENCES_LEFT = 410
 
 def preferences_dialog():
 
-
     dialog = Gtk.Dialog(_("Editor Preferences"), None,
                     Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                     (_("Cancel"), Gtk.ResponseType.REJECT,
@@ -48,10 +47,7 @@ def preferences_dialog():
     edit_prefs_panel, edit_prefs_widgets = _edit_prefs_panel()
     playback_prefs_panel, playback_prefs_widgets  = _playback_prefs_panel()
     view_pres_panel, view_pref_widgets = _view_prefs_panel()
-    # Jan-2017 - SvdB
     performance_panel, performance_widgets = _performance_panel()
-    # Apr-2017 - SvdB
-    #shortcuts_panel, shortcuts_widgets = _shortcuts_panel()
 
     notebook = Gtk.Notebook()
     notebook.set_size_request(PREFERENCES_WIDTH, PREFERENCES_HEIGHT)
@@ -62,15 +58,18 @@ def preferences_dialog():
     notebook.append_page(performance_panel, Gtk.Label(label=_("Performance")))
     guiutils.set_margins(notebook, 4, 24, 6, 0)
 
-    dialog.connect('response', _preferences_dialog_callback, (gen_opts_widgets, edit_prefs_widgets, playback_prefs_widgets, view_pref_widgets, \
-        performance_widgets))
+    dialog.connect('response', _preferences_dialog_callback, 
+                    (gen_opts_widgets, edit_prefs_widgets, playback_prefs_widgets, 
+                    view_pref_widgets, performance_widgets))
     dialog.vbox.pack_start(notebook, True, True, 0)
     dialogutils.set_outer_margins(dialog.vbox)
     dialogutils.default_behaviour(dialog)
-    # Jul-2016 - SvdB - The next line is to get rid of the message "GtkDialog mapped without a transient parent. This is discouraged."
     dialog.set_transient_for(gui.editor_window.window)
     dialog.show_all()
 
+    notebook.set_current_page(0) # gen_opts_widgets
+
+
 def _preferences_dialog_callback(dialog, response_id, all_widgets):
     if response_id == Gtk.ResponseType.ACCEPT:
         editorpersistance.update_prefs_from_widgets(all_widgets)
@@ -98,7 +97,7 @@ def _general_options_panel():
     for profile in profiles:
         default_profile_combo.append_text(profile[0])
     default_profile_combo.set_active(mltprofiles.get_default_profile_index())
-    spin_adj = Gtk.Adjustment(value=prefs.undos_max, lower=editorpersistance.UNDO_STACK_MIN, upper=editorpersistance.UNDO_STACK_MAX, step_incr=1)
+    spin_adj = Gtk.Adjustment(value=prefs.undos_max, lower=editorpersistance.UNDO_STACK_MIN, upper=editorpersistance.UNDO_STACK_MAX, step_increment=1)
     undo_max_spin = Gtk.SpinButton.new_with_range(editorpersistance.UNDO_STACK_MIN, editorpersistance.UNDO_STACK_MAX, 1)
     undo_max_spin.set_adjustment(spin_adj)
     undo_max_spin.set_numeric(True)
@@ -165,7 +164,7 @@ def _edit_prefs_panel():
     prefs = editorpersistance.prefs
 
     # Widgets
-    spin_adj = Gtk.Adjustment(value=prefs.default_grfx_length, lower=1, upper=15000, step_incr=1)
+    spin_adj = Gtk.Adjustment(value=prefs.default_grfx_length, lower=1, upper=15000, step_increment=1)
     gfx_length_spin = Gtk.SpinButton()
     gfx_length_spin.set_adjustment(spin_adj)
     gfx_length_spin.set_numeric(True)
@@ -201,7 +200,16 @@ def _edit_prefs_panel():
     if hasattr(prefs, 'hide_file_ext'):
         hide_file_ext_button.set_active(prefs.hide_file_ext)
 
-    # Layout
+    auto_render_plugins = Gtk.CheckButton()
+    auto_render_plugins.set_active(prefs.auto_render_media_plugins)
+
+    dnd_action = Gtk.ComboBoxText()
+    dnd_action.append_text(_("Always Overwrite Blanks"))
+    dnd_action.append_text(_("Overwrite Blanks on non-V1 Tracks"))
+    dnd_action.append_text(_("Always Insert"))
+    dnd_action.set_active(editorpersistance.prefs.dnd_action) # appconsts values correspond with order here.
+
+    row17 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Drag-and-Drop Action:")), dnd_action, PREFERENCES_LEFT))
     row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Graphics default length:")), gfx_length_spin, PREFERENCES_LEFT))
     row9 = _row(guiutils.get_checkbox_row_box(cover_delete, Gtk.Label(label=_("Cover Transition/Fade clips on delete if possible"))))
     # Jul-2016 - SvdB - For play_pause button
@@ -209,15 +217,17 @@ def _edit_prefs_panel():
     row13 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Mouse Horizontal Scroll Direction:")), hor_scroll_dir, PREFERENCES_LEFT))
     row12 = _row(guiutils.get_checkbox_row_box(hide_file_ext_button, Gtk.Label(label=_("Hide file extensions when importing Clips"))))
     row15 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Open Clip in Effects Editor")), effects_editor_clip_load, PREFERENCES_LEFT))
-    # Apr-2017 - SvdB - For Fast Forward / Reverse options
-
+    row16 = _row(guiutils.get_checkbox_row_box(auto_render_plugins, Gtk.Label(label=_("Autorender Generators"))))
+    
     vbox = Gtk.VBox(False, 2)
+    vbox.pack_start(row17, False, False, 0)
     vbox.pack_start(row4, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
     vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(row13, False, False, 0)
     vbox.pack_start(row12, False, False, 0)
     vbox.pack_start(row15, False, False, 0)
+    vbox.pack_start(row16, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
@@ -226,7 +236,7 @@ def _edit_prefs_panel():
     # Apr-2017 - SvdB - Added ffwd / rev values
     return vbox, (gfx_length_spin, cover_delete,
                   mouse_scroll_action, hide_file_ext_button, hor_scroll_dir,
-                  effects_editor_clip_load)
+                  effects_editor_clip_load, auto_render_plugins, dnd_action)
 
 def _playback_prefs_panel():
     prefs = editorpersistance.prefs
@@ -241,11 +251,9 @@ def _playback_prefs_panel():
     if hasattr(prefs, 'play_pause'):
         play_pause_button.set_active(prefs.play_pause)
 
-# ------------------------------ timeline_start_end_button
     timeline_start_end_button = Gtk.CheckButton()
     if hasattr(prefs, 'timeline_start_end'):
         timeline_start_end_button.set_active(prefs.timeline_start_end)
-# ------------------------------ End of timeline_start_end_button
 
     auto_center_on_updown = Gtk.CheckButton()
     auto_center_on_updown.set_active(prefs.center_on_arrow_move)
@@ -255,25 +263,25 @@ def _playback_prefs_panel():
 
     # Apr-2017 - SvdB - For FF/Rev speed options
     if hasattr(prefs, 'ffwd_rev_shift'):
-        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_shift, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_shift, lower=1, upper=10, step_increment=1)
     else:
-        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_increment=1)
     ffwd_rev_shift_spin = Gtk.SpinButton()
     ffwd_rev_shift_spin.set_adjustment(spin_adj)
     ffwd_rev_shift_spin.set_numeric(True)
 
     if hasattr(prefs, 'ffwd_rev_ctrl'):
-        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_ctrl, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_ctrl, lower=1, upper=10, step_increment=1)
     else:
-        spin_adj = Gtk.Adjustment(value=10, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=10, lower=1, upper=10, step_increment=1)
     ffwd_rev_ctrl_spin = Gtk.SpinButton()
     ffwd_rev_ctrl_spin.set_adjustment(spin_adj)
     ffwd_rev_ctrl_spin.set_numeric(True)
 
     if hasattr(prefs, 'ffwd_rev_caps'):
-        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_caps, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_caps, lower=1, upper=10, step_increment=1)
     else:
-        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_incr=1)
+        spin_adj = Gtk.Adjustment(value=1, lower=1, upper=10, step_increment=1)
     ffwd_rev_caps_spin = Gtk.SpinButton()
     ffwd_rev_caps_spin.set_adjustment(spin_adj)
     ffwd_rev_caps_spin.set_numeric(True)
@@ -281,15 +289,10 @@ def _playback_prefs_panel():
     loop_clips = Gtk.CheckButton()
     loop_clips.set_active(prefs.loop_clips)
 
-    # Layout
     row2 = _row(guiutils.get_checkbox_row_box(auto_center_on_stop, Gtk.Label(label=_("Center Current Frame on Playback Stop"))))
     row13 = _row(guiutils.get_checkbox_row_box(auto_center_on_updown, Gtk.Label(label=_("Center Current Frame after Up/Down Arrow"))))
-    # Jul-2016 - SvdB - For play_pause button
     row10 = _row(guiutils.get_checkbox_row_box(play_pause_button, Gtk.Label(label=_("Enable single Play/Pause button"))))
-    # Apr-2017 - SvdB - For Fast Forward / Reverse options
-# ------------------------------ timeline_start_end_button
     row11 = _row(guiutils.get_checkbox_row_box(timeline_start_end_button, Gtk.Label(label=_("Enable To Start and To End buttons"))))
-# ------------------------------ End of timeline_start_end_button
     row14 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Shift Key:")), ffwd_rev_shift_spin, PREFERENCES_LEFT))
     row14.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Shift Key is held (Only using KEYS).\n" \
         "Enabling multiple modifier keys will multiply the set values.\n" \
@@ -308,12 +311,8 @@ def _playback_prefs_panel():
     vbox.pack_start(row18, False, False, 0)
     vbox.pack_start(row2, False, False, 0)
     vbox.pack_start(row13, False, False, 0)
-    # Jul-2016 - SvdB - For play_pause button
     vbox.pack_start(row10, False, False, 0)
-# ------------------------------ timeline_start_end_button
     vbox.pack_start(row11, False, False, 0)
-# ------------------------------ End of timeline_start_end_button
-    # Apr-2017 - SvdB - For ffwd / rev speed
     vbox.pack_start(row14, False, False, 0)
     vbox.pack_start(row15, False, False, 0)
     vbox.pack_start(row16, False, False, 0)
@@ -322,16 +321,9 @@ def _playback_prefs_panel():
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    # Jul-2016 - SvdB - Added play_pause_button
-    # Apr-2017 - SvdB - Added ffwd / rev values
-# ------------------------------ timeline_start_end_button
     return vbox, (auto_center_on_stop,
                   play_pause_button, timeline_start_end_button, auto_center_on_updown,
                   ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
-# ------------------------------ End of timeline_start_end_button
-#    return vbox, (auto_center_on_stop,
-#                  play_pause_button, auto_center_on_updown,
-#                  ffwd_rev_shift_spin, ffwd_rev_ctrl_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
 
 def _view_prefs_panel():
     prefs = editorpersistance.prefs
@@ -367,31 +359,10 @@ def _view_prefs_panel():
     show_full_file_names = Gtk.CheckButton()
     show_full_file_names.set_active(prefs.show_full_file_names)
 
-    buttons_combo = Gtk.ComboBoxText()
-    buttons_combo.append_text(_("Glass"))
-    buttons_combo.append_text(_("Simple"))
-    buttons_combo.append_text(_("No Decorations"))
-    buttons_combo.set_active( prefs.buttons_style )
-
-    dark_combo = Gtk.ComboBoxText()
-    dark_combo.append_text(_("Flowblade Theme"))
-    dark_combo.append_text(_("Dark Theme"))
-    dark_combo.append_text(_("Light Theme"))
-    dark_combo.set_active(prefs.theme)
-
-
-    theme_combo = Gtk.ComboBoxText()
-    for theme in gui._THEME_COLORS:
-        theme_combo.append_text(theme[4])
-    theme_combo.set_active(prefs.theme_fallback_colors)
-
-    audio_levels_combo = Gtk.ComboBoxText()
-    audio_levels_combo.append_text(_("Display All Levels"))
-    audio_levels_combo.append_text(_("Display Levels On Request"))
-    if prefs.display_all_audio_levels == True:
-        audio_levels_combo.set_active(0)
-    else:
-        audio_levels_combo.set_active(1)
+    # --------------------------------- Colorized icons
+    colorized_icons = Gtk.CheckButton()
+    colorized_icons.set_active(prefs.colorized_icons)
+    # ------------------------------------ End of Colorized icons
 
     window_mode_combo = Gtk.ComboBoxText()
     window_mode_combo.append_text(_("Single Window"))
@@ -407,58 +378,55 @@ def _view_prefs_panel():
     # Aug-2019 - SvdB - BB
     tracks_combo.set_active(prefs.double_track_hights)
 
+    # DEPRECATED, remove.
     top_row_layout = Gtk.ComboBoxText()
     top_row_layout.append_text(_("3 panels if width (1450px+) available"))
     top_row_layout.append_text(_("2 panels always"))
     top_row_layout.set_active(prefs.top_row_layout)
 
-    monitors_data = utils.get_display_monitors_size_data()
+    monitors_data = utilsgtk.get_display_monitors_size_data()
     layout_monitor = Gtk.ComboBoxText()
-    combined_w, combined_h = monitors_data[0]
+
+    combined_w, combined_h = utilsgtk.get_combined_monitors_size()
+            
     layout_monitor.append_text(_("Full Display area: ") + str(combined_w) + " x " + str(combined_h))
-    if len(monitors_data) >= 3:
-        for monitor_index in range(1, len(monitors_data)):
+    if len(monitors_data) >= 2:
+        for monitor_index in range(0, len(monitors_data)):
             monitor_w, monitor_h = monitors_data[monitor_index]
             layout_monitor.append_text(_("Monitor ") + str(monitor_index) + ": " + str(monitor_w) + " x " + str(monitor_h))
     layout_monitor.set_active(prefs.layout_display_index)
 
-
-    # Layout
     row00 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Application window mode:")), window_mode_combo, PREFERENCES_LEFT))
-    #row0 = _row(guiutils.get_checkbox_row_box(force_english_check, Gtk.Label(label=_("Use English texts on localized OS"))))
     row9 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Force Language:")), force_language_combo, PREFERENCES_LEFT))
     row1 = _row(guiutils.get_checkbox_row_box(display_splash_check, Gtk.Label(label=_("Display splash screen"))))
-    row2 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Buttons style:")), buttons_combo, PREFERENCES_LEFT))
-    row3 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme request, icons and colors:")), dark_combo, PREFERENCES_LEFT))
-    row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme detection fail fallback colors:")), theme_combo, PREFERENCES_LEFT))
-    row5 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Default audio levels display:")), audio_levels_combo, PREFERENCES_LEFT))
+    #row3 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme request, icons and colors:")), dark_combo, PREFERENCES_LEFT))
+    #row4 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Theme detection fail fallback colors:")), theme_combo, PREFERENCES_LEFT))
     row7 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Tracks Heights:")), tracks_combo, PREFERENCES_LEFT))
     # Feb-2017 - SvdB - For full file names
     row6 =  _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
-    row8 =  _row(guiutils.get_two_column_box(Gtk.Label(label=_("Top row layout:")), top_row_layout, PREFERENCES_LEFT))
-
     row10 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Do GUI layout based on:")), layout_monitor, PREFERENCES_LEFT))
-
+    row11 =  _row(guiutils.get_checkbox_row_box(colorized_icons, Gtk.Label(label=_("Toolbar color icons"))))
+    
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row00, False, False, 0)
     vbox.pack_start(row10, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
     vbox.pack_start(row1, False, False, 0)
-    vbox.pack_start(row2, False, False, 0)
-    vbox.pack_start(row3, False, False, 0)
-    vbox.pack_start(row4, False, False, 0)
-    vbox.pack_start(row5, False, False, 0)
+    #vbox.pack_start(row3, False, False, 0)
+    #vbox.pack_start(row4, False, False, 0)
     vbox.pack_start(row7, False, False, 0)
     # Feb-2017 - SvdB - For full file names
     vbox.pack_start(row6, False, False, 0)
-    vbox.pack_start(row8, False, False, 0)
+    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
-
+    
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    # Feb-2017 - SvdB - Added code for full file names
-    return vbox, (force_language_combo, display_splash_check, buttons_combo, dark_combo, theme_combo, audio_levels_combo,
-                  window_mode_combo, show_full_file_names, tracks_combo, top_row_layout, layout_monitor)
+    return vbox, (force_language_combo, display_splash_check, window_mode_combo, show_full_file_names,
+                  tracks_combo, top_row_layout, layout_monitor, colorized_icons)
+
+
+
 
 def _performance_panel():
     # Jan-2017 - SvdB
@@ -472,7 +440,7 @@ def _performance_panel():
     warning_icon = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.DIALOG)
     warning_label = Gtk.Label(label=_("Changing these values may cause problems with playback and rendering.\nThe safe values are Render Threads:1, Allow Frame Dropping: No."))
 
-    spin_adj = Gtk.Adjustment(value=prefs.perf_render_threads, lower=1, upper=multiprocessing.cpu_count(), step_incr=1)
+    spin_adj = Gtk.Adjustment(value=prefs.perf_render_threads, lower=1, upper=multiprocessing.cpu_count(), step_increment=1)
     perf_render_threads = Gtk.SpinButton(adjustment=spin_adj)
     #perf_render_threads.set_adjustment(spin_adj)
     perf_render_threads.set_numeric(True)
