@@ -23,7 +23,10 @@ Module contains objects used to capture project data.
 """
 import cairo
 import datetime
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import hashlib
 import os
 
@@ -181,11 +184,20 @@ class Project:
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
                 continue
+            if media_file.container_data != None:
+                continue
             if file_path == media_file.path:
                 return True
 
         return False
 
+    def get_bin_for_media_file_id(self, media_file_id):
+        for bin in self.bins:
+            for file_id in bin.file_ids:
+                if file_id == media_file_id:
+                    return bin
+        return None
+        
     def get_media_file_for_path(self, file_path):
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
@@ -197,12 +209,21 @@ class Project:
     def get_media_file_for_second_path(self, file_path):
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
-                continue
-            print(file_path, media_file.path, media_file.second_file_path)
-            
+                continue            
             if file_path == media_file.second_file_path:
                 return media_file
         return None
+
+    def get_media_file_for_container_data(self, container_data):
+        for key, media_file in list(self.media_files.items()):
+            if media_file.type == appconsts.PATTERN_PRODUCER:
+                continue
+            if media_file.container_data == None:
+                continue
+            if media_file.container_data.program == container_data.program and \
+                media_file.container_data.unrendered_media == container_data.unrendered_media:
+                return media_file
+        return None
         
     def delete_media_file_from_current_bin(self, media_file):
         global media_files_changed_since_last_save
@@ -239,8 +260,9 @@ class Project:
         self.add_named_sequence(name)
         
     def add_named_sequence(self, name):
-        seq = sequence.Sequence(self.profile, name)
+        seq = sequence.Sequence(self.profile, appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK)
         seq.create_default_tracks()
+        seq.name = name
         self.sequences.append(seq)
         self.next_seq_number += 1
 
@@ -500,7 +522,9 @@ class Thumbnailer:
         producer = mlt.Producer(self.profile, str(file_path))
         if producer.is_valid() == False:
             raise ProducerNotValidError(file_path)
-
+        #if producer.get("seekable") == "0": lets see about this
+        #    raise ProducerNotValidError("Video file not seekable, cannot be edited. Transcode to another format.\n\n" + file_path)
+            
         info = utils.get_file_producer_info(producer)
 
         length = producer.get_length()
@@ -517,8 +541,9 @@ class Thumbnailer:
         # This is used for audio files which don't need a thumbnail written
         # but do need file length known
 
-        # Create one frame producer
         producer = mlt.Producer(self.profile, str(file_path))
+        if producer.get("seekable") == "0":
+            raise ProducerNotValidError("Audio file not seekable, cannot be edited.\n\n" + file_path)
         return producer.get_length()
 
 
@@ -564,7 +589,6 @@ def get_default_project():
     """
     profile = mltprofiles.get_default_profile()
     project = Project(profile)
-    project.c_seq.compositing_mode = editorpersistance.prefs.default_compositing_mode
     return project
 
 
