@@ -18,7 +18,11 @@
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-import mlt7 as mlt
+import copy
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import locale
 import os
 import subprocess
@@ -28,7 +32,7 @@ import threading
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('PangoCairo', '1.0')
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, Gio
 from gi.repository import GLib
 
 import appconsts
@@ -37,17 +41,16 @@ import editorstate
 import editorpersistance
 import gui
 import guiutils
-import mltenv
-import mltprofiles
-import mlttransitions
-import mltfilters
+import mltinit
 import patternproducer
 import persistance
+import pickle
 import processutils
 import respaths
 import renderconsumer
 import translations
 import userfolders
+import utils
 
 """
 This module implements media import from another project feature.
@@ -57,6 +60,7 @@ There is so much bolierplate needed for this feature that it was best to create
 """
 
 MEDIA_ASSETS_IMPORT_FILE = "media_assets_import_file"
+GENERATORS_IMPORT_FILE = "generators_import_file"
 
 _info_window = None
 _media_paths_written_to_disk_complete_callback = None
@@ -67,37 +71,47 @@ class ProjectLoadThread(threading.Thread):
         self.filename = filename
 
     def run(self):
-        Gdk.threads_enter()
-        _info_window.info.set_text("Loading project " + self.filename + "...")
-        Gdk.threads_leave()
-
+        GLib.timeout_add(0, self._update_info_window)
+        
         persistance.show_messages = False
         target_project = persistance.load_project(self.filename, False, True)
         
         target_project.c_seq = target_project.sequences[target_project.c_seq_index]
 
-        # Media file media assets
+        # Media file media assets and generator assets are handled a differently.
         media_assets = ""
+        generator_assets = []
+
         for media_file_id, media_file in target_project.media_files.items():
             if isinstance(media_file, patternproducer.AbstractBinClip):
                 continue
-            if os.path.isfile(media_file.path):                
+            if  media_file.container_data != None:
+                # Generator clone
+                generator_assets.append(copy.deepcopy(media_file.container_data))
+            elif os.path.isfile(media_file.path):
+                # File clone
                 media_assets = media_assets + str(media_file.path) + "\n"
 
         with atomicfile.AtomicFileWriter(_get_assets_file(), "w") as afw:
             f = afw.get_file()
             f.write(media_assets)
 
+        with atomicfile.AtomicFileWriter(_get_generators_file(), "wb") as afw:
+            write_file = afw.get_file()
+            pickle.dump(generator_assets, write_file)
+
         _shutdown()
 
-
+    def _update_info_window(self):
+        _info_window.info.set_text("Loading project  " + self.filename + "...\nPlease wait...")
+        
 class ProcesslauchThread(threading.Thread):
     def __init__(self, filename):
         threading.Thread.__init__(self)
         self.filename = filename
 
     def run(self):
-        write_files(self.filename)
+        _write_files(self.filename)
 
 
 # ----------------------------------------------------------- interface
@@ -117,21 +131,24 @@ def get_imported_media():
     files_list = [x.rstrip("\n") for x in files_list] 
     return files_list
     
-def write_files(filename):
+def get_imported_generators():
+    return utils.unpickle(_get_generators_file())
+
+# ----------------------------------------------------------- data gathering process launch and callback to import files
+def _write_files(filename):
     print("Starting media import...")
     FLOG = open(userfolders.get_cache_dir() + "log_media_import", 'w')
     p = subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowblademediaimport", filename], stdin=FLOG, stdout=FLOG, stderr=FLOG)
     p.wait()
     
-    GLib.idle_add(assets_write_complete)
+    GLib.idle_add(_assets_write_complete)
 
-def assets_write_complete():
+def _assets_write_complete():
     _media_paths_written_to_disk_complete_callback()
 
-
 # ------------------------------------------------------------ module internal
 def _do_assets_write(filename):
-    _create_info_dialog()
+    #_create_info_dialog()
     
     global load_thread
     load_thread = ProjectLoadThread(filename)
@@ -140,6 +157,9 @@ def _do_assets_write(filename):
 def _get_assets_file():
     return userfolders.get_cache_dir() + MEDIA_ASSETS_IMPORT_FILE
 
+def _get_generators_file():
+    return userfolders.get_cache_dir() + GENERATORS_IMPORT_FILE
+    
 def _create_info_dialog():
     dialog = Gtk.Window(Gtk.WindowType.TOPLEVEL)
     dialog.set_title(_("Loading Media Import Project"))
@@ -149,17 +169,12 @@ def _create_info_dialog():
     status_box.pack_start(info_label, False, False, 0)
     status_box.pack_start(Gtk.Label(), True, True, 0)
 
-    progress_bar = Gtk.ProgressBar()
-    progress_bar.set_fraction(0.2)
-    progress_bar.set_pulse_step(0.1)
-
     est_box = Gtk.HBox(False, 2)
     est_box.pack_start(Gtk.Label(label=""),False, False, 0)
     est_box.pack_start(Gtk.Label(), True, True, 0)
 
     progress_vbox = Gtk.VBox(False, 2)
     progress_vbox.pack_start(status_box, False, False, 0)
-    progress_vbox.pack_start(progress_bar, True, True, 0)
     progress_vbox.pack_start(est_box, False, False, 0)
 
     alignment = guiutils.set_margins(progress_vbox, 12, 12, 12, 12)
@@ -169,8 +184,6 @@ def _create_info_dialog():
     dialog.set_position(Gtk.WindowPosition.CENTER)
     dialog.show_all()
 
-    # Make refs available for updates
-    dialog.progress_bar = progress_bar
     dialog.info = info_label
 
     global _info_window
@@ -194,47 +207,34 @@ def main(root_path, filename):
     # Set paths.
     respaths.set_paths(root_path)
 
-    # Load editor prefs and list of recent projects
+    # Load editor prefs.
     editorpersistance.load()
-    
-    # Init translations module with translations data
-    translations.init_languages()
-    translations.load_filters_translations()
-    mlttransitions.init_module()
 
-    # Init gtk threads
-    Gdk.threads_init()
-    Gdk.threads_enter()
+    # Create app.
+    global _app
+    _app = ProjectImportApp()
+    _app.filename = filename
+    _app.run(None)
 
-    # Themes
-    if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
-        Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
-        if editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME \
-            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_GRAY \
-            or editorpersistance.prefs.theme == appconsts.FLOWBLADE_THEME_NEUTRAL:
-            gui.apply_gtk_css(editorpersistance.prefs.theme)
-            
-    repo = mlt.Factory().init()
-    processutils.prepare_mlt_repo(repo)
-    
-    # Set numeric locale to use "." as radix, MLT initilizes this to OS locale and this causes bugs 
-    locale.setlocale(locale.LC_NUMERIC, 'C')
 
-    # Check for codecs and formats on the system
-    mltenv.check_available_features(repo)
-    renderconsumer.load_render_profiles()
+class ProjectImportApp(Gtk.Application):
+    def __init__(self, *args, **kwargs):
+        Gtk.Application.__init__(self, application_id=None,
+                                 flags=Gio.ApplicationFlags.FLAGS_NONE)
+        self.connect("activate", self.on_activate)
 
-    # Load filter and compositor descriptions from xml files.
-    mltfilters.load_filters_xml(mltenv.services)
-    mlttransitions.load_compositors_xml(mltenv.transitions)
+    def on_activate(self, data=None):
+        # Themes
+        gui.apply_theme(editorpersistance.prefs.theme)
 
-    # Create list of available mlt profiles
-    mltprofiles.load_profile_list()
+        # Init mlt.
+        repo = mltinit.init_with_translations()
+        
+        _create_info_dialog()
+        
+        GLib.idle_add(_do_assets_write, self.filename)
 
-    GLib.idle_add(_do_assets_write, filename)
-    
-    Gtk.main()
-    Gdk.threads_leave()
-    
+        self.add_window(_info_window)
+        
 def _shutdown():
-    Gtk.main_quit()
+    _app.quit()
