@@ -32,9 +32,7 @@ import gui
 import guicomponents
 import guiutils
 from editorstate import current_sequence
-import mltprofiles
 import renderconsumer
-import respaths
 import utils
 
 destroy_window_event_id = -1
@@ -217,7 +215,7 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
     
     label = Gtk.Label(label=_("Speed %:"))
 
-    adjustment = Gtk.Adjustment(value=float(100), lower=float(1), upper=float(2900), step_incr=float(1))
+    adjustment = Gtk.Adjustment(value=float(100), lower=float(1), upper=float(2900), step_increment=float(1))
     fb_widgets.adjustment = adjustment
 
     spin = Gtk.SpinButton()
@@ -260,9 +258,9 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
     hbox.pack_start(slider_hbox, False, False, 0)
 
     profile_selector = ProfileSelector()
-    profile_selector.fill_options()
+    profile_selector.set_initial_selection()
     profile_selector.widget.set_sensitive(True)
-    fb_widgets.out_profile_combo = profile_selector.widget
+    fb_widgets.categories_combo = profile_selector.categories_combo
 
     quality_selector = RenderQualitySelector()
     fb_widgets.quality_cb = quality_selector.widget
@@ -287,7 +285,7 @@ def show_slowmo_dialog(media_file, default_range_render, _response_callback):
     vbox.pack_start(guiutils.pad_label(24, 12), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target File:")), name_row, 120), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Folder:")), fb_widgets.out_folder, 120), False, False, 0)
-    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Profile:")), fb_widgets.out_profile_combo, 200), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Profile:")), fb_widgets.categories_combo.widget, 200), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Encoding:")), fb_widgets.encodings_cb, 200), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Quality:")), fb_widgets.quality_cb, 200), False, False, 0)
     vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
@@ -363,7 +361,7 @@ def show_reverse_dialog(media_file, default_range_render, _response_callback):
     
     label = Gtk.Label(label=_("Speed %:"))
 
-    adjustment = Gtk.Adjustment(value=float(-100), lower=float(-600), upper=float(-1), step_incr=float(1))
+    adjustment = Gtk.Adjustment(value=float(-100), lower=float(-600), upper=float(-1), step_increment=float(1))
     fb_widgets.hslider = Gtk.HScale()
     fb_widgets.hslider.set_adjustment(adjustment)
     fb_widgets.hslider.set_draw_value(False)
@@ -385,9 +383,9 @@ def show_reverse_dialog(media_file, default_range_render, _response_callback):
     hbox.pack_start(slider_hbox, False, False, 0)
 
     profile_selector = ProfileSelector()
-    profile_selector.fill_options()
+    profile_selector.set_initial_selection()
     profile_selector.widget.set_sensitive(True)
-    fb_widgets.out_profile_combo = profile_selector.widget
+    fb_widgets.categories_combo = profile_selector.categories_combo
 
     quality_selector = RenderQualitySelector()
     fb_widgets.quality_cb = quality_selector.widget
@@ -434,7 +432,7 @@ def show_reverse_dialog(media_file, default_range_render, _response_callback):
     vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target File:")), name_row, 120), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Folder:")), fb_widgets.out_folder, 120), False, False, 0)
-    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Profile:")), fb_widgets.out_profile_combo, 200), False, False, 0)
+    vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Profile:")), fb_widgets.categories_combo.widget, 200), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Encoding:")), fb_widgets.encodings_cb, 200), False, False, 0)
     vbox.pack_start(guiutils.get_two_column_box(Gtk.Label(label=_("Target Quality:")), fb_widgets.quality_cb, 200), False, False, 0)
     vbox.pack_start(guiutils.pad_label(18, 12), False, False, 0)
@@ -506,11 +504,9 @@ class RenderAudioRateSelector():
 class RenderEncodingSelector():
 
     def __init__(self, quality_selector, extension_label, audio_desc_label):
-        self.widget = Gtk.ComboBoxText()
-        for encoding in renderconsumer.encoding_options:
-            self.widget.append_text(encoding.name)
-            
-        self.widget.set_active(0)
+        self.categorised_combo = guicomponents.get_encodings_combo()
+        self.widget = self.categorised_combo.widget
+        self.categorised_combo.set_selected(renderconsumer.DEFAULT_ENCODING_NAME)
         self.widget.connect("changed", 
                             lambda w,e: self.encoding_selection_changed(), 
                             None)
@@ -521,15 +517,21 @@ class RenderEncodingSelector():
         self.audio_desc_label = audio_desc_label
         
     def encoding_selection_changed(self):
-        enc_index = self.widget.get_active()
-        
-        self.quality_selector.update_quality_selection(enc_index)
-        
-        encoding = renderconsumer.encoding_options[enc_index]
-        self.extension_label.set_text("." + encoding.extension)
+        try:
+            name, encoding = self.categorised_combo.get_selected()
+            enc_index = renderconsumer.get_encoding_index(encoding)
+            self.quality_selector.update_quality_selection(enc_index)
+            
+            self.extension_label.set_text("." + encoding.extension)
 
-        if self.audio_desc_label != None:
-            self.audio_desc_label.set_markup(encoding.get_audio_description())
+            if self.audio_desc_label != None:
+                self.audio_desc_label.set_markup(encoding.get_audio_description())
+        except:
+            pass # this gets called too early on start-up
+
+    def get_selected_encoding_index(self):
+        name, encoding = self.categorised_combo.get_selected()
+        return renderconsumer.get_encoding_index(encoding)
 
 
 class PresetEncodingsSelector():
@@ -547,19 +549,15 @@ class PresetEncodingsSelector():
 
 class ProfileSelector():
     def __init__(self, out_profile_changed_callback=None):
-        self.widget = Gtk.ComboBoxText() # filled later when current sequence known
+        self.categories_combo = guicomponents.get_profiles_combo()
+        self.widget = self.categories_combo.widget
         if out_profile_changed_callback != None:
-            self.widget.connect('changed', lambda w:  out_profile_changed_callback())
+            self.widget.connect('changed', lambda w:  out_profile_changed_callback(self.categories_combo))
         self.widget.set_sensitive(False)
         self.widget.set_tooltip_text(_("Select render profile"))
         
-    def fill_options(self):
-        self.widget.get_model().clear()
-        self.widget.append_text(current_sequence().profile.description())
-        profiles = mltprofiles.get_profiles()
-        for profile in profiles:
-            self.widget.append_text(profile[0])
-        self.widget.set_active(0)
+    def set_initial_selection(self):
+        self.categories_combo.set_selected(current_sequence().profile.description())
 
 
 class ProfileInfoBox(Gtk.VBox):
@@ -584,57 +582,66 @@ def get_range_selection_combo():
     return range_cb
 
 # ------------------------------------------------------------ panels
-def get_render_panel_left(render_widgets):
-    small_height = editorstate.screen_size_small_height()
-    
+def get_render_panel_left(render_widgets, render_clicked_cb, to_queue_clicked_cb):
     file_opts_panel = guiutils.get_named_frame(_("File"), render_widgets.file_panel.vbox, 4)         
     profile_panel = guiutils.get_named_frame(_("Render Profile"), render_widgets.profile_panel.vbox, 4)
-
-    if small_height == False:
-        encoding_panel = guiutils.get_named_frame(_("Encoding Format"), render_widgets.encoding_panel.vbox, 4)
-    
-    render_type_panel = guiutils.get_named_frame(_("Render Type"), render_widgets.render_type_panel.vbox, 4)
+    encoding_panel = guiutils.get_named_frame(_("Encoding Format"), render_widgets.encoding_panel.vbox, 4)
     
     render_panel = Gtk.VBox()
     render_panel.pack_start(file_opts_panel, False, False, 0)
-    render_panel.pack_start(render_type_panel, False, False, 0)
     render_panel.pack_start(profile_panel, False, False, 0)
-    if small_height == False:
+    if editorstate.screen_size_small_height() == False:
+        render_range_panel = guiutils.get_named_frame(_("Range"), render_widgets.render_range_panel.vbox, 4)
         render_panel.pack_start(encoding_panel, False, False, 0)
-        render_panel.pack_start(Gtk.Label(), True, True, 0)
-    elif editorstate.SCREEN_HEIGHT == 900: # 900px height screens need most small height fixes but not this
-        encoding_panel = guiutils.get_named_frame(_("Encoding Format"), render_widgets.encoding_panel.vbox, 4)
+        if editorstate.screen_size_large_width() == True:
+            render_panel.pack_start(render_widgets.args_panel.box, False, False, 0)  
+            
+        render_panel.pack_start(render_range_panel, False, False, 0)
+
+        if editorstate.screen_size_large_width() == True:
+
+            render_widgets.queue_button.connect("clicked", 
+                                                 to_queue_clicked_cb, 
+                                                 None)
+
+            render_widgets.render_button.connect("clicked", 
+                                                 render_clicked_cb, 
+                                                 None)
+            buttons_panel = Gtk.HBox()
+            buttons_panel.pack_start(Gtk.Label(), True, True, 0)
+            buttons_panel.pack_start(render_widgets.queue_button, False, False, 0)
+            buttons_panel.pack_start(render_widgets.render_button, False, False, 0)                                         
+
+            render_panel.pack_start(Gtk.Label(), True, True, 0)
+            render_panel.pack_start(buttons_panel, False, False, 0)
+        else:
+            render_panel.pack_start(Gtk.Label(), True, True, 0)
+    else:
         render_panel.pack_start(encoding_panel, False, False, 0)
         render_panel.pack_start(Gtk.Label(), True, True, 0)
     
+    # Stop panel **width** popping when/if GPU renders added.
+    render_panel.set_size_request(300, 30)
+    
     return render_panel
 
 def get_render_panel_right(render_widgets, render_clicked_cb, to_queue_clicked_cb):
-
-
-    if editorstate.SCREEN_HEIGHT < 900:
-        encoding_panel = guiutils.get_named_frame(_("Encoding Format"), render_widgets.encoding_panel.vbox, 4)
-
-    small_height = (editorstate.SCREEN_HEIGHT < 902)
+    small_height = editorstate.screen_size_small_height() #(editorstate.SCREEN_HEIGHT < 902)
     
     opts_panel = guiutils.get_named_frame(_("Render Args"), render_widgets.args_panel.vbox, 4)
-    
+
     bin_row = Gtk.HBox()
-    bin_row.pack_start(guiutils.get_pad_label(10, 8),  False, False, 0)
+    bin_row.pack_start(guiutils.get_pad_label(5, 8),  False, False, 0)
     bin_row.pack_start(render_widgets.args_panel.open_in_bin,  False, False, 0)
     bin_row.pack_start(guiutils.get_pad_label(4, 1),  False, False, 0)
     bin_row.pack_start(Gtk.Label(label=_("Open File in Bin")),  False, False, 0)
     bin_row.pack_start(Gtk.Label(), True, True, 0)
     guiutils.set_margins(bin_row, 0,2,0,0)
-
-    range_row = Gtk.HBox()
-    range_row.pack_start(guiutils.get_pad_label(10, 8),  False, False, 0)
-    range_row.pack_start(render_widgets.range_cb,  True, True, 0)
-
+    
     buttons_panel = Gtk.HBox()
-    if not  editorstate.screen_size_small_width():
-        buttons_panel.pack_start(Gtk.Label(), True, True, 0)
-        buttons_panel.pack_start(render_widgets.queue_button, False, False, 0)
+    #if not editorstate.screen_size_small_width():
+    buttons_panel.pack_start(Gtk.Label(), True, True, 0)
+    buttons_panel.pack_start(render_widgets.queue_button, False, False, 0)
     buttons_panel.pack_start(Gtk.Label(), True, True, 0)
     buttons_panel.pack_start(render_widgets.render_button, False, False, 0)
 
@@ -648,22 +655,25 @@ def get_render_panel_right(render_widgets, render_clicked_cb, to_queue_clicked_c
 
     render_panel = Gtk.VBox()
     if small_height:
+        render_range_panel = guiutils.get_named_frame(_("Range"), render_widgets.render_range_panel.vbox, 4)
+        render_panel.pack_start(render_range_panel, False, False, 0)
         if editorstate.SCREEN_HEIGHT < 900:
-            render_panel.pack_start(encoding_panel, False, False, 0)
-        render_panel.pack_start(opts_panel, True, True, 0)
+            render_panel.pack_start(opts_panel, False, False, 0)
+        else:
+            render_panel.pack_start(opts_panel, True, True, 0)
     else:
         render_panel.pack_start(opts_panel, True, True, 0)
     if small_height == False:
-        render_panel.pack_start(guiutils.get_pad_label(10, 22), False, False, 0)
         render_panel.pack_start(bin_row, False, False, 0)
     else:
-        render_panel.pack_start(Gtk.Label(), True, True, 0)
+        render_panel.pack_start(guiutils.get_pad_label(10, 12), False, False, 0)
+        render_panel.pack_start(bin_row, False, False, 0)
+        if editorstate.SCREEN_HEIGHT < 900:
+            render_panel.pack_start(Gtk.Label(), True, True, 0)
     
-    render_panel.pack_start(range_row, False, False, 0)
     if small_height == False:
         render_panel.pack_start(guiutils.get_pad_label(10, 12), False, False, 0)
-    else:
-        render_panel.pack_start(guiutils.get_pad_label(10, 4), False, False, 0)
+
     render_panel.pack_start(buttons_panel, False, False, 0)
 
     return render_panel
@@ -697,6 +707,14 @@ class RenderFilePanel():
         self.movie_name.set_tooltip_text(_("Give name for rendered file"))
 
 
+class RenderRangePanel():
+    
+    def __init__(self, range_selector):
+        self.vbox = Gtk.VBox(False, 2)
+        self.vbox.pack_start(range_selector, False, False, 0)
+
+
+"""
 class RenderTypePanel():
     
     def __init__(self, render_type_changed_callback, preset_selection_changed_callback):
@@ -715,6 +733,7 @@ class RenderTypePanel():
                                                          self.type_combo, 80), 
                                                          False, False, 0)
         self.vbox.pack_start(self.presets_selector.widget, False, False, 0)
+"""
 
 class RenderProfilePanel():
 
@@ -752,7 +771,7 @@ class RenderProfilePanel():
     def use_project_check_toggled(self, checkbutton):
         self.out_profile_combo.widget.set_sensitive(checkbutton.get_active() == False)
         if checkbutton.get_active() == True:
-            self.out_profile_combo.widget.set_active(0)
+            self.out_profile_combo.categories_combo.set_selected(current_sequence().profile.description())
         
 
 class RenderEncodingPanel():
@@ -795,6 +814,7 @@ class RenderArgsPanel():
     def __init__(self, save_args_callback, 
                  load_args_callback, display_selection_callback,
                  set_default_values_callback):
+
         self.load_args_callback = load_args_callback
         self.save_args_callback = save_args_callback
         self.display_selection_callback = display_selection_callback
@@ -830,12 +850,12 @@ class RenderArgsPanel():
         sw = Gtk.ScrolledWindow()
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         sw.add(self.opts_view)
-        sw.set_size_request(*FFMPEG_VIEW_SIZE)
 
         scroll_frame = Gtk.Frame()
         scroll_frame.add(sw)
 
         self.hamburger_launch = guicomponents.HamburgerPressLaunch(self.hamburger_launch_pressed)
+        self.hamburger_launch.connect_launched_menu(_hamburger_menu)
         guiutils.set_margins(self.hamburger_launch.widget,5,0,0,0)
 
         opts_buttons_row = Gtk.HBox(False)
@@ -914,11 +934,207 @@ class RenderArgsPanel():
             self.ext_entry.set_text("")
 
 
+class RenderArgsRow():
+
+    def __init__(self, save_args_callback, 
+                 load_args_callback, display_selection_callback,
+                 set_default_values_callback):
+
+        self.load_args_callback = load_args_callback
+        self.save_args_callback = save_args_callback
+        self.display_selection_callback = display_selection_callback
+        self.set_default_values_callback = set_default_values_callback
+
+        self.args_text = ""
+        self.ext = ""
+        
+        self.cancel_text  = ""
+        self.cancel_extension = ""
+        
+        # ----------------------------------------- Render panel row
+        self.use_args_label = Gtk.Label(label=_("Render using args"))
+        self.text_buffer = None
+        
+        self.use_args_check = Gtk.CheckButton()
+        self.use_args_check.connect("toggled", self.use_args_toggled)
+        self.use_args_check.set_tooltip_text(_("Render using key=value rendering options"))
+        self.use_args_check.set_margin_left(2)
+        
+        self.args_info = Gtk.Label()
+        self.args_info.set_sensitive(False)
+        self.args_info.set_ellipsize(Pango.EllipsizeMode.END)
+        
+        self.open_in_bin = Gtk.CheckButton()
+
+        surface_active = guiutils.get_cairo_image("kb_configuration")
+        surface_not_active = guiutils.get_cairo_image("kb_configuration_not_active")
+        surfaces = [surface_active, surface_not_active]
+        self.args_edit_launch = guicomponents.HamburgerPressLaunch(lambda w,e:self.args_edit(), surfaces)
+        self.args_edit_launch .set_sensitive(False)
+        
+        use_opts_row = Gtk.HBox()
+        use_opts_row.pack_start(self.use_args_check,  False, False, 0)
+        use_opts_row.pack_start(guiutils.get_pad_label(4, 1), False, False, 0)
+        use_opts_row.pack_start(self.use_args_label, False, False, 0)
+        use_opts_row.pack_start(guiutils.get_pad_label(4, 1), False, False, 0)
+        use_opts_row.pack_start(self.args_info, True, True, 0)
+        use_opts_row.pack_start(self.args_edit_launch.widget, False, False, 0)
+        guiutils.set_margins(use_opts_row, 2,2,0,0)
+        
+        # ----------------------------------------- Popover pane
+        self.load_selection_button = Gtk.Button(label=_("Load Args From Current Encoding"))
+        self.load_selection_button.connect("clicked", lambda w: self.display_selection_callback())
+        self.load_selection_row = guiutils.get_left_justified_box([self.load_selection_button])
+        guiutils.set_margins(self.load_selection_row, 0,4,0,0)
+        
+        self.opts_view = Gtk.TextView()
+        self.opts_view.set_sensitive(True)
+        self.opts_view.set_pixels_above_lines(2)
+        self.opts_view.set_left_margin(2)
+        self.opts_view.set_tooltip_text(_("Edit render options"))
+        
+        sw = Gtk.ScrolledWindow()
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.add(self.opts_view)
+        sw.set_size_request(418, 400)
+        
+        scroll_frame = Gtk.Frame()
+        scroll_frame.add(sw)
+
+        self.opts_load_button = Gtk.Button(label=_("Load Args"))
+        self.opts_load_button.connect("clicked", lambda w: load_args_callback())
+    
+        self.opts_save_button = Gtk.Button(label=_("Save Args"))
+        self.opts_save_button.connect("clicked", lambda w: save_args_callback())
+
+        self.ext_label = Gtk.Label(label=_("Ext.:"))
+        self.ext_label.set_sensitive(True)
+
+        self.ext_entry = Gtk.Entry()
+        self.ext_entry.set_width_chars(5)    
+        self.ext_entry.set_sensitive(True)
+
+        self.save_load_row = Gtk.HBox(False, 2)
+        self.save_load_row.pack_start(self.opts_save_button, False, False, 0)
+        self.save_load_row.pack_start(self.opts_load_button, False, False, 0)
+        self.save_load_row.pack_start(Gtk.Label(), True, True, 0)
+        self.save_load_row.pack_start(self.ext_label, False, False, 0)
+        self.save_load_row.pack_start(self.ext_entry, False, False, 0)
+        guiutils.set_margins(self.save_load_row, 4,0,0,0)
+
+        self.cancel_button = Gtk.Button(label=_("Cancel"))
+        self.cancel_button.connect("clicked", lambda w: self.cancel())
+    
+        self.set_args_button = Gtk.Button(label=_("Set Args"))
+        self.set_args_button.connect("clicked", lambda w: self.set_args())
+
+        self.set_args_row = Gtk.HBox(False, 2)
+        self.set_args_row.pack_start(Gtk.Label(), True, True, 0)
+        self.set_args_row.pack_start(self.cancel_button, False, False, 0)
+        self.set_args_row.pack_start(self.set_args_button, False, False, 0)
+        guiutils.set_margins(self.set_args_row, 24,0,0,0)
+        
+        self.pop_over_pane = Gtk.VBox(False, 2)
+        self.pop_over_pane.pack_start(self.load_selection_row, False, False, 0)
+        self.pop_over_pane.pack_start(scroll_frame, False, False, 0)
+        self.pop_over_pane.pack_start(self.save_load_row, False, False, 0)
+        self.pop_over_pane.pack_start(self.set_args_row, False, False, 0)
+        guiutils.set_margins(self.pop_over_pane, 4,4,4,4)
+        self.pop_over_pane.show_all()
+        
+        # ----------------------------------------- build GUI
+        self.box = Gtk.VBox(False, 2)
+        self.box.pack_start(use_opts_row , False, False, 0)
+
+        self.args_popover = Gtk.Popover.new(self.args_edit_launch.widget)
+        self.args_popover.add(self.pop_over_pane)
+
+    def args_edit(self):
+        self.init_popover_with_render_data()
+        self.save_cancel_data()
+        self.args_popover.show()
+
+    def set_args(self):
+        self.args_info.set_text(self.info_text)
+        self.save_render_data()
+        self.args_popover.popdown()
+        
+    def cancel(self):
+        self.opts_view.get_buffer().set_text(self.cancel_text)
+        self.ext_entry.set_text(self.cancel_extension)
+        self.args_popover.popdown()
+
+    def save_render_data(self):
+        buf = self.opts_view.get_buffer()
+        text = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), include_hidden_chars=True)
+        self.args_text = text
+        self.ext = self.ext_entry.get_text()
+        
+    def clear_cancel_data(self):
+        self.cancel_text = ""
+        self.cancel_extension = ""
+
+    def save_cancel_data(self):
+        buf = self.opts_view.get_buffer()
+        text = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), include_hidden_chars=True)
+        self.cancel_text = text
+        self.cancel_extension = self.ext_entry.get_text()
+
+    def display_encoding_args(self, profile, enc_index, qual_index):
+        encoding_option = renderconsumer.encoding_options[enc_index]
+        quality_option = encoding_option.quality_options[qual_index]
+        args_vals_list = encoding_option.get_args_vals_tuples_list(profile, quality_option)
+        text = ""
+        info_text = ""
+        for arg_val in args_vals_list:
+            k, v = arg_val
+            line = str(k) + "=" + str(v)
+            text = text + line  + "\n"
+            if info_text == "":
+                info_text = info_text + line + ",..."
+        
+        self.info_text = info_text # This is used to update self.args_info text when exiting with 'Set Args'.
+        
+        text_buffer = Gtk.TextBuffer()
+        text_buffer.set_text(text)
+        self.opts_view.set_buffer(text_buffer)            
+        self.ext_entry.set_text(encoding_option.extension)
+
+        # Save cancel data on first show.
+        if self.cancel_text == "":
+            self.cancel_text = text
+            self.cancel_extension = encoding_option.extension
+
+    def init_popover_with_render_data(self):
+        self.opts_view.get_buffer().set_text(self.args_text)
+        self.ext_entry.set_text(self.ext)
+        self.args_info.set_text(self.info_text)
+                
+    def use_args_toggled(self, checkbutton):
+        active = checkbutton.get_active()
+        if active == True:
+            if self.args_text == "":
+                self.display_selection_callback()
+                self.save_render_data()
+                self.init_popover_with_render_data()
+            else:
+                self.init_popover_with_render_data()
+            
+            self.save_cancel_data()
+            self.args_popover.show()
+            self.args_edit_launch.set_sensitive(True)
+        else:
+            self.opts_view.set_buffer(Gtk.TextBuffer())
+            self.ext_entry.set_text("")
+            self.args_info.set_text("")
+            self.args_edit_launch.set_sensitive(False)
+
+
 class RenderArgsPanelSmall():
 
     def __init__(self, save_args_callback, 
                  load_args_callback, display_selection_callback):
-                 
+
         self.display_selection_callback = display_selection_callback
         
         self.args_edit_window = None
@@ -943,7 +1159,7 @@ class RenderArgsPanelSmall():
 
         self.open_in_bin = Gtk.CheckButton()
 
-        self.open_args_editor_button = Gtk.Button(_("Edit Args:"))
+        self.open_args_editor_button = Gtk.Button(label=_("Edit Args:"))
         self.open_args_editor_button.connect("clicked", lambda w: self.open_edit_window())
         self.open_args_editor_button.set_sensitive(False)
 
@@ -1033,7 +1249,6 @@ class RenderArgsPanelSmall():
 class RenderArgsEditWindow(Gtk.Window):
     def __init__(self, args_panel):
         GObject.GObject.__init__(self)
-        #self.connect("delete-event", lambda w, e:close_audio_monitor())
         
         self.args_panel = args_panel
 
@@ -1042,17 +1257,16 @@ class RenderArgsEditWindow(Gtk.Window):
         self.opts_view.set_pixels_above_lines(2)
         self.opts_view.set_left_margin(2)
         self.opts_view.set_buffer(self.args_panel.text_buffer)
-
+        self.opts_view.set_size_request(300, 400)
+        
         sw = Gtk.ScrolledWindow()
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         sw.add(self.opts_view)
-        sw.set_size_request(*FFMPEG_VIEW_SIZE)
 
         scroll_frame = Gtk.Frame()
         scroll_frame.add(sw)
-        scroll_frame.set_size_request(200, 300)
 
-        self.load_selection_button = Gtk.Button(_("Load Selection"))
+        self.load_selection_button = Gtk.Button(label=_("Load Selection"))
         self.load_selection_button.connect("clicked", lambda w: self.args_panel.display_selection_callback())
 
         self.ext_label = Gtk.Label(label=_("Ext.:"))
@@ -1088,6 +1302,8 @@ class RenderArgsEditWindow(Gtk.Window):
         pane.pack_start(guiutils.get_pad_label(4, 12), False, True, 0)
         pane.pack_start(editor_buttons_row, False, True, 0)
 
+        pane.set_size_request(300, 400)
+            
         align = guiutils.set_margins(pane, 12, 12, 12, 12)
         # Set pane and show window
         self.add(align)
@@ -1095,5 +1311,5 @@ class RenderArgsEditWindow(Gtk.Window):
         self.show_all()
         self.set_resizable(False)
         self.set_keep_above(True) # Perhaps configurable later
-    
+
     
