@@ -22,8 +22,10 @@
 Module contains class Sequence that is the multitrack media object being edited
 by the application. A project has 1-n of these.
 """
-import time
-import mlt7 as mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
 
 import appconsts
@@ -33,7 +35,7 @@ import mltfilters
 import mlttransitions
 import mltrefhold
 import patternproducer
-import tlinerender
+# import tlinerender
 import utils
 
 # Media types for tracks or clips
@@ -97,10 +99,10 @@ class Sequence:
     """
     Multitrack MLT object
     """
-    def __init__(self, profile, name="sequence"):
+    def __init__(self, profile, initial_comp_mode):
 
         # Data members
-        self.name = name # name of sequence
+        self.name = "sequence" # initial name of sequence
         self.next_id = 0 # id for next created clip
         self.profile = profile
         self.master_audio_gain = 1.0
@@ -113,11 +115,11 @@ class Sequence:
         self.watermark_filter = None
         self.watermark_file_path = None
         self.seq_len = 0 # used in trim crash hack, remove when fixed
-        self.compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
+        self.compositing_mode = initial_comp_mode
 
         # MLT objects for a multitrack sequence
         self.init_mlt_objects()
-
+        
     # ----------------------------------- mlt init
     def init_mlt_objects(self):
         # MLT objects for multitrack sequence
@@ -204,6 +206,10 @@ class Sequence:
         self.tracks[0].clips.append(black_track_clip) # py
         self.tracks[0].append(black_track_clip, 0, 0) # mlt
 
+        # Create fulltrack compositors if needed.
+        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+            self.add_full_track_compositors()
+            
     def _create_black_track_clip(self):
         # Create 1 fr long black bg clip and set in and out
         global black_track_clip
@@ -371,10 +377,14 @@ class Sequence:
         pan_filter.set("start", value)
         track.attach(pan_filter)
         track.pan_filter = pan_filter 
-
+            
     def set_track_pan_value(self, track, value):
-        track.pan_filter.set("start", str(value))
-        track.audio_pan = value
+        if hasattr(track, "pan_filter") == False or track.pan_filter == None:
+            self.add_track_pan_filter(track, value)
+            track.audio_pan = value
+        else:
+            track.pan_filter.set("start", str(value))
+            track.audio_pan = value
     
     def remove_track_pan_filter(self, track):
         # This method is used for master too, and called with tractor then
@@ -434,6 +444,8 @@ class Sequence:
         if ttl != None:
             producer.set("ttl", str(ttl))
 
+        producer.set("mute_on_pause", str(1))
+
         return producer
 
     def create_slowmotion_producer(self, path, speed):
@@ -700,7 +712,7 @@ class Sequence:
         for i in range(self.first_video_index, len(self.tracks) - 1):
             track = self.tracks[i]
         
-            compositor = self.create_compositor("##affineblend")
+            compositor = self.create_compositor("##blend")
 
             a_track = self.first_video_index
             b_track = track.id 
@@ -853,7 +865,7 @@ class Sequence:
         if seq_len < 1:
             seq_len = 1
         
-        tlinerender.get_renderer().update_hidden_track(self.tracks[-1], seq_len)
+        # tlinerender.get_renderer().update_hidden_track(self.tracks[-1], seq_len)
 
     def fix_v1_for_render(self): 
         # This is a workaround to fix Issue #941 with H248 encoder not being able handle 
@@ -1046,7 +1058,85 @@ class Sequence:
                 cut_frame = prev_cut_frame
             
         return cut_frame
-    
+
+    def find_next_editable_clip_and_track(self, tline_frame):
+        """
+        Returns next selectable clip and track.
+        """
+        cut_frame = -1
+        clip_track = None
+        select_clip = None
+        for i in range(1, len(self.tracks) - 1):
+            track = self.tracks[i]
+            
+            # Get index and clip
+            index = track.get_clip_index_at(tline_frame)
+            try:
+                clip = track.clips[index]
+                clip_start_in_tline = track.clip_start(index)
+                # We are looking for media clips only and after tline_frame.
+                while clip.is_blanck_clip == True or clip_start_in_tline < tline_frame:
+                    clip = track.clips[index + 1]
+                    clip_start_in_tline = track.clip_start(index + 1)
+                    index = index + 1
+            except Exception as e:
+                continue # No selectable clip on track after frame
+
+            # Get next cut frame
+            clip_start_in_tline = track.clip_start(index)
+            length = clip.clip_out - clip.clip_in 
+            next_cut_frame = clip_start_in_tline + length + 1 # +1 clip out inclusive
+                     
+            # Set cut frame
+            if cut_frame == -1 and next_cut_frame > tline_frame:
+                cut_frame = next_cut_frame
+                clip_track = track
+                select_clip = clip
+            elif next_cut_frame < cut_frame and next_cut_frame > tline_frame:
+                cut_frame = next_cut_frame
+                clip_track = track
+                select_clip = clip
+
+        return (select_clip, clip_track)
+
+    def find_prev_editable_clip_and_track(self, tline_frame):
+        """
+        Returns prev selectable clip and track.
+        """
+        cut_frame = -1
+        clip_track = None
+        select_clip = None
+        for i in range(1, len(self.tracks) - 1):
+            track = self.tracks[i]
+            
+            # Get index and clip
+            index = track.get_clip_index_at(tline_frame)
+            try:
+                clip = track.clips[index]
+                clip_start_in_tline = track.clip_start(index)
+                # We are looking for media clips only and before tline_frame.
+                while clip.is_blanck_clip == True or clip_start_in_tline > tline_frame:
+                    clip = track.clips[index - 1]
+                    clip_start_in_tline = track.clip_start(index - 1)
+                    index = index - 1
+            except Exception as e:
+                continue # No selectable clip on track before frame
+
+            # Get prev cut frame
+            prev_cut_frame = clip_start_in_tline
+                     
+            # Set cut frame
+            if cut_frame == -1 and prev_cut_frame < tline_frame:
+                cut_frame = prev_cut_frame
+                clip_track = track
+                select_clip = clip
+            elif prev_cut_frame > cut_frame and prev_cut_frame < tline_frame:
+                cut_frame = prev_cut_frame
+                clip_track = track
+                select_clip = clip
+
+        return (select_clip, clip_track)
+
     def get_closest_cut_frame(self, track_id, frame):
         track = self.tracks[track_id]
         index = track.get_clip_index_at(frame)
@@ -1161,6 +1251,24 @@ class Sequence:
                 if clip.is_blanck_clip == False:
                     clip.waveform_data = None
 
+    # ----------------------------------- tracks auto expand feature
+    def get_tracks_clips_counts(self):
+        clips_counts = []
+        for i in range(1, len(self.tracks) - 1): # hidden tracks
+            clips_counts.append(len(self.tracks[i].clips))
+ 
+        return clips_counts
+    
+    def get_inital_drop_target_track(self, tracks_clips_count_before):
+        current_tracks_clips_count = self.get_tracks_clips_counts()
+        for i in range(0, len(current_tracks_clips_count)):
+            if tracks_clips_count_before[i] == 0:
+                if current_tracks_clips_count[i] > 0:
+                    return i + 1 # lists don't include bottom bg track.
+
+        return None
+
+    # ----------------------------------- info pronts
     def print_all(self):
         print("------------------------######")
         for i in range(0, len(self.tracks)):
@@ -1199,35 +1307,7 @@ def get_media_type(file_path):
     Returns media type of file.
     """
     return utils.get_media_type(file_path)
-    """
-    if os.path.exists(file_path):
-        mime_type = utils.get_file_type(file_path)
-    else:
-        # IMAGE_SEQUENCE media objects have a MLT formatted resource path that does not
-        # point to an existing file in the file system. 
-        # We're doing a heuristic here to identify those.
-        pros_index = file_path.find("%0")
-        d_index = file_path.find("d.")
-        if pros_index != -1 and d_index != -1:
-            return IMAGE_SEQUENCE
-        all_index = file_path.find(".all")
-        if all_index != -1:
-            return IMAGE_SEQUENCE
-            
-        return FILE_DOES_NOT_EXIST
-        
-    if mime_type.startswith("video"):
-        return VIDEO
-    
-    if mime_type.startswith("audio"):
-        return AUDIO
-    
-    if mime_type.startswith("image"):
-        return IMAGE
-    
-    return UNKNOWN
-    """
-    
+
 def _clip_length(clip):
     return clip.clip_out - clip.clip_in + 1
 
@@ -1236,27 +1316,25 @@ def _sort_compositors_comparator(a_comp):
 
 # ----------------------------- sequence cloning for tracks count change
 def create_sequence_clone_with_different_track_count(old_seq, v_tracks, a_tracks):
-    # Create new sequence with different number of tracks
+    # Create new sequence with different number of tracks.
     global AUDIO_TRACKS_COUNT, VIDEO_TRACKS_COUNT
     AUDIO_TRACKS_COUNT = a_tracks
     VIDEO_TRACKS_COUNT = v_tracks
-    new_seq = Sequence(old_seq.profile, old_seq.name)
+    new_seq = Sequence(old_seq.profile, old_seq.compositing_mode)
     new_seq.create_default_tracks()
-
-    # Clone track clips from old sequence to clone sequence
+    new_seq.name = old_seq.name
+ 
+    # Clone track clips from old sequence to clone sequence.
     if old_seq.first_video_index - 1 > a_tracks:
         _clone_for_fewer_tracks(old_seq, new_seq)
     else:
         _clone_for_more_tracks(old_seq, new_seq)
 
-    # Clone compositors from old seq to new to correct tracks on new seq
+    # Clone compositors from old seq to new to correct tracks on new seq.
     track_delta = new_seq.first_video_index - old_seq.first_video_index
     new_seq.clone_compositors_from_sequence(old_seq, track_delta)
-
-    # Copy modes values
-    new_seq.compositing_mode = old_seq.compositing_mode
         
-    # copy next clip id data
+    # Copy next clip id data.
     new_seq.next_id = old_seq.next_id
     return new_seq
         
