@@ -27,14 +27,11 @@ except ImportError:
 import gi
 
 gi.require_version('Gtk', '3.0')
-from gi.repository import GLib
 
-import locale
-import mlt7 as mlt
-import os
-import pickle
-import subprocess
-import sys
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import threading
 import time
 
@@ -42,7 +39,6 @@ import appconsts
 import ccrutils
 import mltheadlessutils
 import mltprofiles
-import processutils
 import renderconsumer
 import toolsencoding
 
