@@ -17,12 +17,13 @@
     You should have received a copy of the GNU General Public License
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
-
+import array
+import cairo
 import copy
 import os
 import pickle
+from PIL import Image, ImageFilter
 import threading
-
 import time
 
 from gi.repository import Gtk, Gdk, GdkPixbuf
@@ -76,6 +77,9 @@ ALIGN_LEFT = 0
 ALIGN_CENTER = 1
 ALIGN_RIGHT = 2
 
+VERTICAL = 0
+HORIZONTAL = 1
+
 def show_titler():
     global _titler_data
     if _titler_data == None:
@@ -108,9 +112,16 @@ def titler_destroy():
         _titler_data = None
 
 def reset_titler():
-    global _titler_data
-    _titler_data = None
+    global _titler, _keep_titler_data, _titler_data
 
+    if _titler != None:
+        temp_keep_val = _keep_titler_data
+        _keep_titler_data = True
+        titler_destroy()
+        _keep_titler_data = temp_keep_val
+        show_titler()
+    else:
+        _titler_data = None
 
 # ------------------------------------------------------------- data
 class TextLayer:
@@ -130,7 +141,10 @@ class TextLayer:
         self.alignment = ALIGN_LEFT
         self.pixel_size = (100, 100)
         self.spacing = 5
-        
+
+        self.gradient_color_rgba = None
+        self.gradient_direction = VERTICAL
+
         self.outline_on = False
         self.outline_color_rgba = (0.3, 0.3, 0.3, 1.0) 
         self.outline_width = 2
@@ -140,9 +154,9 @@ class TextLayer:
         self.shadow_opacity = 100
         self.shadow_xoff = 3
         self.shadow_yoff = 3
-        self.shadow_blur = 0.0 # not impl yet, for future so that we don't need to break save format again
+        self.shadow_blur = 0.0
         
-        self.pango_layout = None # PangoTextLayout(self)
+        self.pango_layout = None # PangoTextLayout object
 
         self.layer_attributes = None # future feature 
         self.visible = True
@@ -186,6 +200,12 @@ class TitlerData:
         for layer in self.layers:
             layer.pango_layout = PangoTextLayout(layer)
 
+    def data_compatibility_update(self):
+        # We added new stuff for 2.8 and need to update data created with older versions.
+        for layer in self.layers:
+            if hasattr(layer, "gradient_color_rgba") == False:
+                layer.gradient_color_rgba = None
+                layer.gradient_direction = VERTICAL
 
 # ---------------------------------------------------------- editor
 class Titler(Gtk.Window):
@@ -211,8 +231,8 @@ class Titler(Gtk.Window):
         
         self.guides_toggle = vieweditor.GuidesViewToggle(self.view_editor)
         
-        add_b = Gtk.Button(_("Add"))
-        del_b = Gtk.Button(_("Delete"))
+        add_b = Gtk.Button(label=_("Add"))
+        del_b = Gtk.Button(label=_("Delete"))
         add_b.connect("clicked", lambda w:self._add_layer_pressed())
         del_b.connect("clicked", lambda w:self._del_layer_pressed())
         add_del_box = Gtk.HBox()
@@ -274,7 +294,7 @@ class Titler(Gtk.Window):
         combo.set_active(0)
         self.font_select = combo
         self.font_select.connect("changed", self._edit_value_changed)
-        adj = Gtk.Adjustment(value=float(DEFAULT_FONT_SIZE), lower=float(1), upper=float(300), step_incr=float(1))
+        adj = Gtk.Adjustment(value=float(DEFAULT_FONT_SIZE), lower=float(1), upper=float(300), step_increment=float(1))
         self.size_spin = Gtk.SpinButton()
         self.size_spin.set_adjustment(adj)
         self.size_spin.connect("changed", self._edit_value_changed)
@@ -336,12 +356,13 @@ class Titler(Gtk.Window):
         buttons_box.pack_start(self.fill_on, False, False, 0)
         buttons_box.pack_start(Gtk.Label(), True, True, 0)
 
-        outline_size = Gtk.Label(_("Size:"))
+        # ------------------------------------------- Outline Panel
+        outline_size = Gtk.Label(label=_("Size:"))
         
         self.out_line_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.3, green=0.3, blue=0.3, alpha=1.0))
         self.out_line_color_button.connect("color-set", self._edit_value_changed)
 
-        adj2 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(50), step_incr=float(1))
+        adj2 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(50), step_increment=float(1))
         self.out_line_size_spin = Gtk.SpinButton()
         self.out_line_size_spin.set_adjustment(adj2)
         self.out_line_size_spin.connect("changed", self._edit_value_changed)
@@ -361,27 +382,29 @@ class Titler(Gtk.Window):
         outline_box.pack_start(self.outline_on, False, False, 0)
         outline_box.pack_start(Gtk.Label(), True, True, 0)
 
-        shadow_opacity_label = Gtk.Label(_("Opacity:"))
-        shadow_xoff = Gtk.Label(_("X Off:"))
-        shadow_yoff = Gtk.Label(_("Y Off:"))
+        # -------------------------------------------- Shadow panel 
+        shadow_opacity_label = Gtk.Label(label=_("Opacity:"))
+        shadow_xoff = Gtk.Label(label=_("X Off:"))
+        shadow_yoff = Gtk.Label(label=_("Y Off:"))
+        shadow_blur_label = Gtk.Label(label=_("Blur:"))
         
         self.shadow_opa_spin = Gtk.SpinButton()
  
-        adj3 = Gtk.Adjustment(value=float(100), lower=float(1), upper=float(100), step_incr=float(1))
+        adj3 = Gtk.Adjustment(value=float(100), lower=float(1), upper=float(100), step_increment=float(1))
         self.shadow_opa_spin.set_adjustment(adj3)
         self.shadow_opa_spin.connect("changed", self._edit_value_changed)
         self.shadow_opa_spin.connect("key-press-event", self._key_pressed_on_widget)
 
         self.shadow_xoff_spin = Gtk.SpinButton()
 
-        adj4 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_incr=float(1))
+        adj4 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_increment=float(1))
         self.shadow_xoff_spin.set_adjustment(adj4)
         self.shadow_xoff_spin.connect("changed", self._edit_value_changed)
         self.shadow_xoff_spin.connect("key-press-event", self._key_pressed_on_widget)
 
         self.shadow_yoff_spin = Gtk.SpinButton()
 
-        adj5 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_incr=float(1))
+        adj5 = Gtk.Adjustment(value=float(3), lower=float(1), upper=float(100), step_increment=float(1))
         self.shadow_yoff_spin.set_adjustment(adj5)
         self.shadow_yoff_spin.connect("changed", self._edit_value_changed)
         self.shadow_yoff_spin.connect("key-press-event", self._key_pressed_on_widget)
@@ -393,6 +416,11 @@ class Titler(Gtk.Window):
         self.shadow_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.3, green=0.3, blue=0.3, alpha=1.0))
         self.shadow_color_button.connect("color-set", self._edit_value_changed)
 
+        self.shadow_blur_spin = Gtk.SpinButton()
+        adj6 = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(20), step_increment=float(1))
+        self.shadow_blur_spin.set_adjustment(adj6)
+        self.shadow_blur_spin.connect("changed", self._edit_value_changed)
+
         shadow_box_1 = Gtk.HBox()
         shadow_box_1.pack_start(shadow_opacity_label, False, False, 0)
         shadow_box_1.pack_start(self.shadow_opa_spin, False, False, 0)
@@ -410,12 +438,47 @@ class Titler(Gtk.Window):
         shadow_box_2.pack_start(shadow_yoff, False, False, 0)
         shadow_box_2.pack_start(self.shadow_yoff_spin, False, False, 0)
         shadow_box_2.pack_start(Gtk.Label(), True, True, 0)
+
+        shadow_box_3 = Gtk.HBox()
+        shadow_box_3.pack_start(shadow_blur_label, False, False, 0)
+        shadow_box_3.pack_start(self.shadow_blur_spin, False, False, 0)
+        shadow_box_3.pack_start(Gtk.Label(), True, True, 0)
+
+        # ------------------------------------ Gradient panel
+        self.gradient_color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(red=0.0, green=0.0, blue=0.8, alpha=1.0))
+        self.gradient_color_button.connect("color-set", self._edit_value_changed)
+        self.gradient_on = Gtk.CheckButton()
+        self.gradient_on.set_active(True)
+        self.gradient_on.connect("toggled", self._edit_value_changed)
+
+        direction_label = Gtk.Label(label=_("Gradient Direction:"))
+        self.direction_combo = Gtk.ComboBoxText()
+        self.direction_combo.append_text(_("Vertical"))
+        self.direction_combo.append_text(_("Horizontal"))
+        self.direction_combo.set_active(0)
+        self.direction_combo.connect("changed", self._edit_value_changed)
+         
+        gradient_box_row1 = Gtk.HBox()
+        gradient_box_row1.pack_start(self.gradient_on, False, False, 0)
+        gradient_box_row1.pack_start(self.gradient_color_button, False, False, 0)
+        gradient_box_row1.pack_start(Gtk.Label(), True, True, 0)
+
+        gradient_box_row2 = Gtk.HBox()
+        gradient_box_row2.pack_start(direction_label, False, False, 0)
+        gradient_box_row2.pack_start(self.direction_combo, False, False, 0)
+        gradient_box_row2.pack_start(Gtk.Label(), True, True, 0)
         
-        load_layers = Gtk.Button(_("Load Layers"))
+        gradient_box = Gtk.VBox()
+        gradient_box.pack_start(gradient_box_row1, False, False, 0)
+        gradient_box.pack_start(gradient_box_row2, False, False, 0)
+        gradient_box.pack_start(Gtk.Label(), True, True, 0)
+                
+        # ---------------------------------------------------- Save and Load buttons        
+        load_layers = Gtk.Button(label=_("Load Layers"))
         load_layers.connect("clicked", lambda w:self._load_layers_pressed())
-        save_layers = Gtk.Button(_("Save Layers"))
+        save_layers = Gtk.Button(label=_("Save Layers"))
         save_layers.connect("clicked", lambda w:self._save_layers_pressed())
-        clear_layers = Gtk.Button(_("Clear All"))
+        clear_layers = Gtk.Button(label=_("Clear All"))
         clear_layers.connect("clicked", lambda w:self._clear_layers_pressed())
 
         layers_save_buttons_row = Gtk.HBox()
@@ -423,19 +486,20 @@ class Titler(Gtk.Window):
         layers_save_buttons_row.pack_start(load_layers, False, False, 0)
         layers_save_buttons_row.pack_start(Gtk.Label(), True, True, 0)
 
-        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
+        # ---------------------------------------------------- X, Y pos input
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_increment=float(1))
         self.x_pos_spin = Gtk.SpinButton()
         self.x_pos_spin.set_adjustment(adj)
         self.x_pos_spin.connect("changed", self._position_value_changed)
         self.x_pos_spin.connect("key-press-event", self._key_pressed_on_widget)
 
-        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_increment=float(1))
         self.y_pos_spin = Gtk.SpinButton()
         self.y_pos_spin.set_adjustment(adj)
         self.y_pos_spin.connect("changed", self._position_value_changed)
         self.y_pos_spin.connect("key-press-event", self._key_pressed_on_widget)
 
-        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_incr=float(1))
+        adj = Gtk.Adjustment(value=float(0), lower=float(0), upper=float(3000), step_increment=float(1))
         self.rotation_spin = Gtk.SpinButton()
         self.rotation_spin.set_adjustment(adj)
         self.rotation_spin.connect("changed", self._position_value_changed)
@@ -446,6 +510,7 @@ class Titler(Gtk.Window):
                                        Gtk.IconSize.BUTTON)
         undo_pos.set_image(undo_icon)
 
+        # ------------------------------------------------- Timeline controls
         next_icon = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "next_frame_s.png")
         prev_icon = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "prev_frame_s.png")
         prev_frame = Gtk.Button()
@@ -458,6 +523,7 @@ class Titler(Gtk.Window):
         self.scale_selector = vieweditor.ScaleSelector(self)
         self.view_editor.scale_select = self.scale_selector
 
+        # ------------------------------------------------------ Panels
         timeline_box = Gtk.HBox()
         timeline_box.pack_start(self.tc_display.widget, False, False, 0)
         timeline_box.pack_start(guiutils.pad_label(12, 12), False, False, 0)
@@ -472,10 +538,14 @@ class Titler(Gtk.Window):
         
         positions_box = Gtk.HBox()
         positions_box.pack_start(Gtk.Label(), True, True, 0)
-        positions_box.pack_start(Gtk.Label(label="X:"), False, False, 0)
+        x_label = Gtk.Label(label="X:")
+        x_label.set_margin_right(4)
+        positions_box.pack_start(x_label, False, False, 0)
         positions_box.pack_start(self.x_pos_spin, False, False, 0)
         positions_box.pack_start(guiutils.pad_label(40, 5), False, False, 0)
-        positions_box.pack_start(Gtk.Label(label="Y:"), False, False, 0)
+        y_label = Gtk.Label(label="Y:")
+        y_label.set_margin_right(4)
+        positions_box.pack_start(y_label, False, False, 0)
         positions_box.pack_start(self.y_pos_spin, False, False, 0)
         positions_box.pack_start(guiutils.pad_label(40, 5), False, False, 0)
         positions_box.pack_start(center_h, False, False, 0)
@@ -488,7 +558,6 @@ class Titler(Gtk.Window):
         controls_panel_1.pack_start(layers_save_buttons_row, False, False, 0)
 
         controls_panel_2 = Gtk.VBox()
-        #controls_panel_2.pack_start(scroll_frame, True, True, 0)
         controls_panel_2.pack_start(font_main_row, False, False, 0)
         controls_panel_2.pack_start(buttons_box, False, False, 0)
 
@@ -498,11 +567,16 @@ class Titler(Gtk.Window):
         controls_panel_4 = Gtk.VBox()
         controls_panel_4.pack_start(shadow_box_1, False, False, 0)
         controls_panel_4.pack_start(shadow_box_2, False, False, 0)
+        controls_panel_4.pack_start(shadow_box_3, False, False, 0)
+
+        controls_panel_5 = Gtk.VBox()
+        controls_panel_5.pack_start(gradient_box, False, False, 0)
 
         notebook = Gtk.Notebook()
         notebook.append_page(guiutils.set_margins(controls_panel_2,8,8,8,8), Gtk.Label(label=_("Font")))
         notebook.append_page(guiutils.set_margins(controls_panel_3,8,8,8,8), Gtk.Label(label=_("Outline")))
         notebook.append_page(guiutils.set_margins(controls_panel_4,8,8,8,8), Gtk.Label(label=_("Shadow")))
+        notebook.append_page(guiutils.set_margins(controls_panel_5,8,8,8,8), Gtk.Label(label=_("Gradient")))
         
         controls_panel = Gtk.VBox()
         controls_panel.pack_start(guiutils.get_named_frame(_("Layer Text"), scroll_frame), True, True, 0)
@@ -514,6 +588,7 @@ class Titler(Gtk.Window):
         view_editor_editor_buttons_row.pack_start(positions_box, False, False, 0)
         view_editor_editor_buttons_row.pack_start(Gtk.Label(), True, True, 0)
 
+        # ------------------------------------------------------- Editor buttons
         keep_label = Gtk.Label(label=_("Keep Layers When Closed"))
         self.keep_layers_check = Gtk.CheckButton()
         self.keep_layers_check.set_active(_keep_titler_data)
@@ -531,15 +606,18 @@ class Titler(Gtk.Window):
         
         editor_buttons_row = Gtk.HBox()
         editor_buttons_row.pack_start(Gtk.Label(), True, True, 0)
-        editor_buttons_row.pack_start(keep_label, False, False, 0)
         editor_buttons_row.pack_start(self.keep_layers_check, False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(4, 1), False, False, 0)
+        editor_buttons_row.pack_start(keep_label, False, False, 0)
         editor_buttons_row.pack_start(guiutils.pad_label(24, 2), False, False, 0)
-        editor_buttons_row.pack_start(open_label, False, False, 0)
         editor_buttons_row.pack_start(self.open_in_current_check, False, False, 0)
-        editor_buttons_row.pack_start(guiutils.pad_label(24, 2), False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(4, 1), False, False, 0)
+        editor_buttons_row.pack_start(open_label, False, False, 0)
+        editor_buttons_row.pack_start(guiutils.pad_label(32, 2), False, False, 0)
         editor_buttons_row.pack_start(exit_b, False, False, 0)
         editor_buttons_row.pack_start(save_titles_b, False, False, 0)
         
+        # ------------------------------------------------------ window layout
         editor_panel = Gtk.VBox()
         editor_panel.pack_start(self.view_editor, True, True, 0)
         editor_panel.pack_start(timeline_box, False, False, 0)
@@ -559,6 +637,7 @@ class Titler(Gtk.Window):
         self._update_gui_with_active_layer_data()
         self.show_all()
 
+        # -------------------------------------------------- window state listeners
         self.connect("size-allocate", lambda w, e:self.window_resized())
         self.connect("window-state-event", lambda w, e:self.window_resized())
     
@@ -583,7 +662,8 @@ class Titler(Gtk.Window):
         self._activate_layer(0)
         self.layer_list.fill_data_model()
         self.view_editor.edit_area.queue_draw()
-
+        self._select_layer(0)
+        
     def show_current_frame(self):
         frame = PLAYER().current_frame()
         length = PLAYER().producer.get_length()
@@ -662,10 +742,12 @@ class Titler(Gtk.Window):
                 filenames = dialog.get_filenames()
                 load_path = filenames[0]
                 new_data = utils.unpickle(load_path)
+                new_data.data_compatibility_update()
                 global _titler_data
                 _titler_data = new_data
                 self.load_titler_data()
-            except:
+            except Exception as e:
+                print("Titler._load_layers_dialog_callback", e)
                 dialog.destroy()
                 # INFOWINDOW
                 return
@@ -696,7 +778,7 @@ class Titler(Gtk.Window):
             self._update_active_layout()
             return True
 
-            # update layer for enter on x, y, angle
+        # update layer for enter on x, y, angle
         if ((event.keyval == Gdk.KEY_Return) and ((widget == self.x_pos_spin) or
             (widget == self.y_pos_spin) or (widget == self.rotation_spin))):
             self.x_pos_spin.update()
@@ -723,20 +805,22 @@ class Titler(Gtk.Window):
         view_editor_layer.mouse_released_listener  = self._editor_layer_mouse_released
         self.view_editor.edit_layers.append(view_editor_layer)
         
+        layer_index = len(_titler_data.layers) - 1
         self.layer_list.fill_data_model()
-        self._activate_layer(len(_titler_data.layers) - 1, True)
+        self._activate_layer(layer_index, True)
+        self._select_layer(layer_index)
         
     def _del_layer_pressed(self):
         # we always need 1 layer
         if len(_titler_data.layers) < 2:
             return
 
-        #active_index = _titler_data.get_active_layer_index()
         _titler_data.layers.remove(_titler_data.active_layer)
         self.view_editor.edit_layers.remove(self.view_editor.active_layer)
         self.layer_list.fill_data_model()
         self._activate_layer(0)
-
+        self._select_layer(0)
+        
     def _layer_visibility_toggled(self, layer_index):
         toggled_visible = (self.view_editor.edit_layers[layer_index].visible == False)
         self.view_editor.edit_layers[layer_index].visible = toggled_visible
@@ -796,6 +880,7 @@ class Titler(Gtk.Window):
         _titler_data.active_layer = _titler_data.layers[layer_index]
         self._update_gui_with_active_layer_data()
         _titler_data.active_layer.update_pango_layout()
+        self._select_layer(layer_index)
 
     def _activate_layer(self, layer_index, is_new_layer=False):
         global _titler_data
@@ -805,12 +890,17 @@ class Titler(Gtk.Window):
             self._update_gui_with_active_layer_data() # Update GUI with layer data
         else:
             self._update_active_layout_font_properties() # Update layer font properties with current GUI values.
-
+            self._update_gui_with_active_layer_data() # Update GUI with layer data
+            
         _titler_data.active_layer.update_pango_layout()
         self.view_editor.activate_layer(layer_index)
         self.view_editor.active_layer.update_rect = True
         self.view_editor.edit_area.queue_draw()
 
+    def _select_layer(self, layer_index):
+        self.layer_list.treeview.get_selection().select_path(Gtk.TreePath.new_from_indices([layer_index]))
+        self.layer_list.queue_draw()
+
     def _editor_layer_mouse_released(self):
         p = self.view_editor.active_layer.edit_point_shape.edit_points[0]
         
@@ -826,6 +916,7 @@ class Titler(Gtk.Window):
 
     def _text_changed(self, widget):
         self._update_active_layout()
+        self._select_layer(_titler_data.get_active_layer_index())
 
     def _position_value_changed(self, widget):
         # mouse release when layer is moved causes this method to be called,
@@ -903,14 +994,25 @@ class Titler(Gtk.Window):
         # SHADOW
         color = self.shadow_color_button.get_color()
         r, g, b = utils.hex_to_rgb(color.to_string())
-        a = self.shadow_opa_spin.get_value() / 100.0
         new_color3 = (r/65535.0, g/65535.0, b/65535.0)  
         _titler_data.active_layer.shadow_color_rgb = new_color3
         _titler_data.active_layer.shadow_on = self.shadow_on.get_active()
         _titler_data.active_layer.shadow_opacity = self.shadow_opa_spin.get_value()
         _titler_data.active_layer.shadow_xoff = self.shadow_xoff_spin.get_value()
         _titler_data.active_layer.shadow_yoff = self.shadow_yoff_spin.get_value()
-                
+        _titler_data.active_layer.shadow_blur = self.shadow_blur_spin.get_value()
+        
+        # GRADIENT
+        if self.gradient_on.get_active() == True:
+            color = self.gradient_color_button.get_color()
+            r, g, b = utils.hex_to_rgb(color.to_string())
+            new_color = (r/65535.0, g/65535.0, b/65535.0, 1.0)  
+            _titler_data.active_layer.gradient_color_rgba = new_color
+        else:
+            _titler_data.active_layer.gradient_color_rgba = None
+        _titler_data.active_layer.gradient_direction = self.direction_combo.get_active() # Combo indexes correspond with values of VERTICAL and HORIZONTAL
+        
+        
         self.view_editor.active_layer.update_rect = True
         _titler_data.active_layer.update_pango_layout()
 
@@ -919,7 +1021,7 @@ class Titler(Gtk.Window):
             return
         
         # This a bit hackish, but works. Finding a method that blocks all
-        # gui events from being added to queue would be nice.
+        # gui events from being added to event queue would be nice.
         self.block_updates = True
         
         # TEXT
@@ -981,7 +1083,20 @@ class Titler(Gtk.Window):
         self.shadow_xoff_spin.set_value(layer.shadow_xoff)
         self.shadow_yoff_spin.set_value(layer.shadow_yoff)
         self.shadow_on.set_active(layer.shadow_on)
+        self.shadow_blur_spin.set_value(layer.shadow_blur)
         
+        # GRADIENT
+        if layer.gradient_color_rgba != None:
+            r, g, b, a = layer.gradient_color_rgba
+            button_color = Gdk.RGBA(r, g, b, 1.0)
+            self.gradient_color_button.set_rgba(button_color)
+            self.gradient_on.set_active(True)
+        else:
+            button_color = Gdk.RGBA(0.0, 0.0, 0.6, 1.0)
+            self.gradient_color_button.set_rgba(button_color)
+            self.gradient_on.set_active(False)
+        self.direction_combo.set_active(layer.gradient_direction)
+                
         self.block_updates = False
 
 
@@ -1002,9 +1117,10 @@ class PangoTextLayout:
         self.font_desc = Pango.FontDescription(layer.get_font_desc_str())
         self.color_rgba = layer.color_rgba
         self.alignment = self._get_pango_alignment_for_layer(layer)
-        self.pixel_size = layer.pixel_size
+        self.pixel_size = layer.pixel_size # this is some placeholder default (100, 100), has no further meaning.
         self.fill_on = layer.fill_on
-        
+        self.gradient_color_rgba = layer.gradient_color_rgba
+
         self.outline_color_rgba = layer.outline_color_rgba
         self.outline_on = layer.outline_on
         self.outline_width = layer.outline_width
@@ -1014,34 +1130,100 @@ class PangoTextLayout:
         self.shadow_opacity = layer.shadow_opacity
         self.shadow_xoff = layer.shadow_xoff
         self.shadow_yoff = layer.shadow_yoff
+        self.shadow_blur = layer.shadow_blur
+        
+        self.gradient_color_rgba = layer.gradient_color_rgba
+        self.gradient_direction = layer.gradient_direction
 
     # called from vieweditor draw vieweditor-> editorlayer->here
-    def draw_layout(self, cr, x, y, rotation, xscale, yscale):
+    def draw_layout(self, cr, x, y, rotation, xscale, yscale, view_editor):
         cr.save()
+
+        fontmap = PangoCairo.font_map_new()
+        context = fontmap.create_context()
+        font_options = cairo.FontOptions()
+        font_options.set_antialias(cairo.Antialias.GOOD)
+        PangoCairo.context_set_font_options(context, font_options)
+        context.changed()
+
+        layout = Pango.Layout.new(context)
         
-        layout = PangoCairo.create_layout(cr)
         layout.set_text(self.text, -1)
         layout.set_font_description(self.font_desc)
         layout.set_alignment(self.alignment)
-    
         self.pixel_size = layout.get_pixel_size()
 
         # Shadow
         if self.shadow_on:
             cr.save()
+
+            # Get colors.
             r, g, b = self.shadow_color_rgb
             a = self.shadow_opacity / 100.0
-            cr.set_source_rgba(r, g, b, a)
-            cr.move_to(x + self.shadow_xoff, y + self.shadow_yoff)
-            cr.scale(xscale, yscale)
-            cr.rotate(rotation)
-            PangoCairo.update_layout(cr, layout)
-            PangoCairo.show_layout(cr, layout)
-            cr.restore()
-        
+
+            # Blurred shadow need s own ImageSurface
+            if self.shadow_blur != 0.0:
+                blurred_img = cairo.ImageSurface(cairo.FORMAT_ARGB32, view_editor.profile_w,  view_editor.profile_h)
+                cr_blurred = cairo.Context(blurred_img)
+                cr_blurred.set_antialias(cairo.Antialias.GOOD)
+                transform_cr = cr_blurred # Set draw transform_cr to cotext for newly created image.
+            else:
+                transform_cr = cr # Set draw transform_cr to out context.
+
+            # Transform and set color.
+            transform_cr.set_source_rgba(r, g, b, a)
+            effective_shadow_xoff = self.shadow_xoff * xscale
+            effective_shadow_yoff = self.shadow_yoff * yscale
+            transform_cr.move_to(x + effective_shadow_xoff, y + effective_shadow_yoff)
+            transform_cr.scale(xscale, yscale)
+            transform_cr.rotate(rotation)
+
+            # If no blur just draw layout on out context.
+            if self.shadow_blur == 0.0:
+                PangoCairo.update_layout(cr, layout)
+                PangoCairo.show_layout(cr, layout)
+                cr.restore()
+            else:
+                # If we have blur - draw shadow, blur it and then draw on out context.
+                PangoCairo.update_layout(cr_blurred, layout)
+                PangoCairo.show_layout(cr_blurred, layout)
+
+                img2 = Image.frombuffer("RGBA", (blurred_img.get_width(), blurred_img.get_height()), blurred_img.get_data(), "raw", "RGBA", 0, 1)
+                effective_blur = xscale * self.shadow_blur # This is not going to be exact
+                                                           # on non-100% scales but let's try to get approximation. 
+                img2 = img2.filter(ImageFilter.GaussianBlur(radius=int(effective_blur)))
+                imgd = img2.tobytes()
+                a = array.array('B',imgd)
+
+                stride = blurred_img.get_width() * 4
+                draw_surface = cairo.ImageSurface.create_for_data (a, cairo.FORMAT_ARGB32,
+                                                              blurred_img.get_width(), blurred_img.get_height(), stride)
+                cr.restore()
+                cr.set_source_surface(draw_surface, 0, 0)
+                cr.paint()
+
         # Text
         if self.fill_on:
-            cr.set_source_rgba(*self.color_rgba)
+            if self.gradient_color_rgba == None:
+                cr.set_source_rgba(*self.color_rgba)
+            else:
+                w, h = self.pixel_size
+                w = float(w) * xscale
+                h = float(h) * yscale
+                if self.gradient_direction == HORIZONTAL:
+                    grad = cairo.LinearGradient (x, 0, x + w, 0)
+                else:
+                    grad = cairo.LinearGradient (0, y, 0, y + h)
+                
+                r, g, b, a = self.color_rgba
+                rg, gg, bg, ag =  self.gradient_color_rgba 
+                    
+                CLIP_COLOR_GRAD_1 = (0,  r, g, b, 1)
+                CLIP_COLOR_GRAD_2 = (1,  rg, gg, bg, 1)
+                grad.add_color_stop_rgba(*CLIP_COLOR_GRAD_1)
+                grad.add_color_stop_rgba(*CLIP_COLOR_GRAD_2)
+                cr.set_source(grad)
+
             cr.move_to(x, y)
             cr.scale(xscale, yscale)
             cr.rotate(rotation)
@@ -1089,7 +1271,7 @@ class TextLayerListView(Gtk.VBox):
         self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
-        self.treeview = Gtk.TreeView(self.storemodel)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(False)
         self.treeview.connect("button-press-event", self.button_press)
@@ -1173,12 +1355,22 @@ class TextLayerListView(Gtk.VBox):
                 visible_icon = self.eye_icon
             else:
                 visible_icon = None 
-            row_data = [self.layer_icon, layer.text, visible_icon]
+            text = self.find_char_in_text(layer.text)
+            row_data = [self.layer_icon, text, visible_icon]
             self.storemodel.append(row_data)
         
         self.scroll.queue_draw()
         _filling_layer_list = False
 
+    def find_char_in_text(self, text):
+        while text.find(" ") == 0 or text.find("\n") == 0:
+            text = text[1:]
+        line_end =text.find("\n")
+        if  line_end != -1:
+            text = text[:line_end]
+        return text
+        
+
 
 class OpenFileThread(threading.Thread):
     
