@@ -29,25 +29,33 @@ gi.require_version('Gtk', '3.0')
 
 from gi.repository import Gtk
 from gi.repository import Gdk
+from gi.repository import GLib
 
 import appconsts
 import dialogs
 import dialogutils
-import edit
+import editorlayout
 import editorpersistance
 import editorstate
-from editorstate import PROJECT
 import gui
 import guiutils
 import modesetting
+import projectdata
 import respaths
-import updater
 
-# New version start-up toolset selections.
-STANDARD_PRESET = 0
-FILM_STYLE_PRESET = 1
-KEEP_EXISTING = 2
 
+# Initial layouts.
+LAYOUT_MONITOR_CENTER = 0
+LAYOUT_MEDIA_PANEL_LEFT = 1
+LAYOUT_MONITOR_LEFT = 2
+LAYOUT_TOP_ROW_FOUR = 3
+
+# Default compositing modes. We have different enums here because of different presentation order.
+COMPOSITING_MODE_STANDARD_FULL_TRACK = 0
+COMPOSITING_MODE_TOP_DOWN_FREE_MOVE = 1
+COMPOSITING_MODE_STANDARD_AUTO_FOLLOW = 2
+
+# Colors
 SELECTED_BG = Gdk.RGBA(0.1, 0.31, 0.58,1.0)
 WHITE_TEXT = Gdk.RGBA(0.9, 0.9, 0.9,1.0)
 DARK_TEXT = Gdk.RGBA(0.1, 0.1, 0.1,1.0)
@@ -79,7 +87,7 @@ def init_data():
                   }
                   
     _TOOL_TIPS =  { appconsts.TLINE_TOOL_INSERT:        _("<b>Left Mouse</b> to move and insert single clip between clips.\n<b>CTRL + Left Mouse</b> to select and move clip range.\n\n<b>Left Mouse</b> on clip ends to trim clip length."),
-                    appconsts.TLINE_TOOL_OVERWRITE:     _("<b>Left Mouse</b> to move clip into new position.\n<b>CTRL + Left Mouse</b> to select and move clip range into new position.\n\n<b>Left Mouse</b> on clip ends to trim clip length."),
+                    appconsts.TLINE_TOOL_OVERWRITE:     _("<b>Left Mouse</b> to select a clip, or to move clip or clip range into new position.\n<b>CTRL + Left Mouse</b> to select a clip range.\n\n<b>Left Mouse</b> near clip's end to trim clip length.<b>\n\nLeft Mouse Drag</b> to draw a box to select a group of clips. Multitrack selection disallows moving clips to another track."),
                     appconsts.TLINE_TOOL_TRIM:          _("<b>Left Mouse</b> to trim closest clip end.\n<b>Left or Right Arrow Key</b> + <b>Enter Key</b> to do the edit using keyboard."), 
                     appconsts.TLINE_TOOL_ROLL:          _("<b>Left Mouse</b> to move closest edit point between 2 clips.\n<b>Left or Right Arrow Key</b> + <b>Enter Key</b> to do the edit using keyboard."), 
                     appconsts.TLINE_TOOL_SLIP:          _("<b>Left Mouse</b> to move clip contents within clip.\n<b>Left or Right Arrow Key</b> + <b>Enter Key</b> to do the edit using keyboard."), 
@@ -87,26 +95,15 @@ def init_data():
                     appconsts.TLINE_TOOL_BOX:           _("<b>1. Left Mouse</b> to draw a box to select a group of clips.\n<b>2. Left Mouse</b> inside the box to move selected clips forward or backward."), 
                     appconsts.TLINE_TOOL_RIPPLE_TRIM:   _("<b>Left Mouse</b> to trim closest clip end and move all clips after it to maintain sync, overwrites not allowed.\n<b>Left or Right Arrow Key</b> + <b>Enter Key</b> to do the edit using keyboard."), 
                     appconsts.TLINE_TOOL_CUT:           _("<b>Left Mouse</b> to cut clip under cursor.\n<b>CTRL + Left Mouse</b> to cut clips on all tracks at cursor position."), 
-                    appconsts.TLINE_TOOL_KFTOOL:        _("Click <b>Left Mouse</b> on Clip to init Volume Keyframe editing, Brightness for media with no audio data.\n<b>Left Mouse</b> to create or drag keyframes.\n<b>Delete Key</b> to delete active Keyframe."),
+                    appconsts.TLINE_TOOL_KFTOOL:        _("Click <b>Left Mouse</b> on Clip to init Volume Keyframe editing, Brightness for media with no audio data.\nUse Hamburger menu to select other values to edit.\n<b>Left Mouse</b> to create or drag keyframes.\n<b>Left Mouse + Control</b> between keyframes drag and set same value for both.\n<b>Delete Key</b> to delete active Keyframe."),
                     appconsts.TLINE_TOOL_MULTI_TRIM:    _("Position cursor near or on clip edges for <b>Trim</b> and <b>Roll</b> edits.\nPosition cursor on clip center for <b>Slip</b> edit.\nDrag with <b>Left Mouse</b> to do edits.\n\n<b>Enter Key</b> to start keyboard edit, <b>Left or Right Arrow Key</b> to move edit point.\n<b>Enter Key</b> to complete keyboard edit.")
                   }
 
-    _PREFS_TOOL_TIPS = {"editorpersistance.prefs.box_for_empty_press_in_overwrite_tool":       _("<b>\n\nLeft Mouse Drag</b> to draw a box to select a group of clips and move\nthe selected clips forward or backward.")}
-    
-    
+
 #----------------------------------------------------- workflow presets
 def _set_workflow_STANDARD():
     editorpersistance.prefs.active_tools = [2, 11, 6, 1, 9, 10] # appconsts.TLINE_TOOL_ID_<X> values
-    editorpersistance.prefs.dnd_action = appconsts.DND_ALWAYS_OVERWRITE
-    editorpersistance.prefs.box_for_empty_press_in_overwrite_tool = True
-    editorpersistance.save()
-
-    modesetting.set_default_edit_mode()
-
-def _set_workflow_FILM_STYLE():
-    editorpersistance.prefs.active_tools = [1, 2, 3, 4, 5, 6, 7]  # appconsts.TLINE_TOOL_ID_<X> values
     editorpersistance.prefs.dnd_action = appconsts.DND_OVERWRITE_NON_V1
-    editorpersistance.prefs.box_for_empty_press_in_overwrite_tool = False
     editorpersistance.save()
 
     modesetting.set_default_edit_mode()
@@ -171,65 +168,16 @@ def _get_image_menu_item(tool_icon_file, text, callback, tool_id):
 def workflow_menu_launched(widget, event):
     guiutils.remove_children(_workflow_menu)
 
-    # ---- preset
-    presets_item = Gtk.MenuItem.new_with_label(_("Workflow Presets"))
-    presets_item.show()
-
-    presets_menu = Gtk.Menu()
-    
-    standard = guiutils.get_menu_item(_("Standard"), _workflow_menu_callback, (None, "preset standard"))
+    standard = guiutils.get_menu_item(_("Reset Tools"), _workflow_menu_callback, (None, "preset standard"))
     standard.show()
-    presets_menu.add(standard)
+    _workflow_menu.add(standard)
 
-    film_style = guiutils.get_menu_item(_("Film Style"), _workflow_menu_callback, (None, "preset filmstyle"))
-    film_style.show()
-    presets_menu.add(film_style)
-    
-    presets_item.set_submenu(presets_menu)
-    _workflow_menu.add(presets_item)
-
-    # --- behaviours
-    guiutils.add_separetor(_workflow_menu)
-
-    behaviours_item = Gtk.MenuItem.new_with_label(_("Behaviours"))
-    behaviours_item.show()
-
-    behaviours_menu = Gtk.Menu()
-
-    dnd_item = Gtk.MenuItem.new_with_label(_("Drag'n'Drop Action"))
-    dnd_item.show()
-
-    dnd_menu = Gtk.Menu()
-    labels = [_("Always Overwrite Blanks"), _("Overwrite Blanks on non-V1 Tracks"), _("Always Insert")]
-    msgs = ["always overwrite", "overwrite nonV1", "always insert"]
-    active_index = editorpersistance.prefs.dnd_action  # appconsts values correspond with order here.
-    _build_radio_menu_items_group(dnd_menu, labels, msgs, _workflow_menu_callback, active_index)
-
-    dnd_item.set_submenu(dnd_menu)
-    behaviours_menu.add(dnd_item)
-
-    default_compositing_item = Gtk.MenuItem.new_with_label(_("New Sequence Default Compositing Mode"))
-    default_compositing_item.show()
-    
-    default_compositing_menu = Gtk.Menu()
-    labels = [_("Top Down Free Move"), _("Top Down Auto Follow"), _("Standard Full Track")]
-    msgs = ["top down", "standard auto", "full_track_auto"]
-    active_index = editorpersistance.prefs.default_compositing_mode  # appconsts values correspond with order here.
-    _build_radio_menu_items_group(default_compositing_menu, labels, msgs, _workflow_menu_callback, active_index)
-
-    default_compositing_item.set_submenu(default_compositing_menu)
-    behaviours_menu.add(default_compositing_item)
-    
     show_tooltips_item = Gtk.CheckMenuItem()
     show_tooltips_item.set_label(_("Show Tooltips for Edit Tools"))
     show_tooltips_item.set_active(editorpersistance.prefs.show_tool_tooltips)
     show_tooltips_item.connect("activate", _workflow_menu_callback, (None, "tooltips"))
     show_tooltips_item.show()
-
-    behaviours_menu.append(show_tooltips_item)
-    
-    behaviours_item.set_submenu(behaviours_menu)
-    _workflow_menu.add(behaviours_item)
+    _workflow_menu.append(show_tooltips_item)
 
     # --- tools
     guiutils.add_separetor(_workflow_menu)
@@ -256,8 +204,6 @@ def workflow_menu_launched(widget, event):
 
 def _get_workflow_tool_menu_item(callback, tool_id, tool_name, tool_icon_file, position):
 
-    tool_active = (tool_id in editorpersistance.prefs.active_tools)
-
     tool_img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + tool_icon_file)
     tool_name_label = Gtk.Label(tool_name)
     
@@ -297,11 +243,6 @@ def _build_radio_menu_items_group(menu, labels, msgs, callback, active_index):
 
 def _get_tooltip_text(tool_id):
     text = _TOOL_TIPS[tool_id]
-    
-    # Add individual extensions based on current prefs
-    if tool_id == appconsts.TLINE_TOOL_OVERWRITE:
-        if editorpersistance.prefs.box_for_empty_press_in_overwrite_tool == True:
-            text += _PREFS_TOOL_TIPS["editorpersistance.prefs.box_for_empty_press_in_overwrite_tool"]
 
     return text
 
@@ -335,15 +276,6 @@ def _get_workflow_tool_submenu(callback, tool_id, position):
 
     sub_menu.add(position_item)
     
-    # Individual prefs for tools
-    if tool_id == appconsts.TLINE_TOOL_OVERWRITE:
-        pref_item = Gtk.CheckMenuItem(_("Do Box Selection and Box Move from empty press"))
-        pref_item.set_active(editorpersistance.prefs.box_for_empty_press_in_overwrite_tool)
-        pref_item.connect("toggled", _TLINE_TOOL_OVERWRITE_box_selection_pref)
-        pref_item.show()
-        sub_menu.add(pref_item)
-        guiutils.add_separetor(sub_menu)
-
     return sub_menu
     
 def _workflow_menu_callback(widget, data):
@@ -357,8 +289,6 @@ def _workflow_menu_callback(widget, data):
         update_tool_dock()
     elif msg == "preset standard":
         _set_workflow_STANDARD()
-    elif msg == "preset filmstyle":
-        _set_workflow_FILM_STYLE()
     elif  msg == "always overwrite":
         editorpersistance.prefs.dnd_action = appconsts.DND_ALWAYS_OVERWRITE
     elif  msg == "overwrite nonV1":
@@ -367,12 +297,6 @@ def _workflow_menu_callback(widget, data):
         editorpersistance.prefs.dnd_action = appconsts.DND_ALWAYS_INSERT
     elif  msg == "tooltips":
         editorpersistance.prefs.show_tool_tooltips = widget.get_active()
-    elif  msg == "top down":
-        editorpersistance.prefs.default_compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
-    elif  msg == "standard auto":
-        editorpersistance.prefs.default_compositing_mode = appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW
-    elif  msg == "standard full":
-        editorpersistance.prefs.default_compositing_mode = appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK
     else:
         try: # Set tool position
             pos = int(msg)
@@ -424,7 +348,13 @@ def _tool_dock_item_press(tool_id, tool_dock_item):
         item.set_item_color(False)
     tool_dock_item.set_item_color(True)
     gui.editor_window.tool_selector_item_activated(None, tool_id)
-    
+
+def set_default_tool_dock_item_selected():
+    for item in dock_items:
+        item.set_item_color(False)
+    dock_items[0].set_item_color(True)
+
+        
 
 class ToolDockItem:
     def __init__(self, kb_shortcut_number, tool_icon_file, tool_name, tool_id):
@@ -439,7 +369,6 @@ class ToolDockItem:
             self.widget.set_tooltip_markup("<b>" + tool_name + " - " + _("Keyboard shortcut: ") + str(kb_shortcut_number) + "</b>" "\n\n" + _get_tooltip_text(tool_id))
         
         self.widget.add(tool_img)
-        #self.set_item_color(True)
 
     def set_item_color(self, selected):
         if selected == True:
@@ -472,144 +401,3 @@ def tline_tool_keyboard_selected(event):
 def select_default_tool():
     tool_id = editorpersistance.prefs.active_tools[0]
     gui.editor_window.change_tool(tool_id)
-        
-# -------------------------------------------------------------- tool prefs
-def _TLINE_TOOL_OVERWRITE_box_selection_pref(check_menu_item):
-    editorpersistance.prefs.box_for_empty_press_in_overwrite_tool = check_menu_item.get_active()
-    editorpersistance.save()
-
-
-
-class WorkflowDialog(Gtk.Dialog):
-
-    def __init__(self):
-        Gtk.Dialog.__init__(self, _("Workflow First Run Wizard"),  gui.editor_window.window,
-                                Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                                (_("Select Preset Workflow and Continue"), Gtk.ResponseType.ACCEPT))
-
-        self.selection = STANDARD_PRESET 
-        
-        info_label_text_1 = _("<b>Welcome to Flowblade</b>")
-        info_label_1 = Gtk.Label(info_label_text_1)
-        info_label_1.set_use_markup(True)
-
-
-        info_label_text_2 = _("<b>Flowblade</b> comes with a configurable workflow.")
-        info_label_2 = Gtk.Label(info_label_text_2)
-        info_label_2.set_use_markup(True)
-
-        INDENT = "    "
-        info_label_text_6 = INDENT + "\u2022" + _(" You can select which <b>Edit Tools</b> you want to use.\n") + \
-                            INDENT + "\u2022" + _(" Many timeline edit <b>behaviours</b> are configurable.\n")
-
-        info_label_6 = Gtk.Label(info_label_text_6)
-        info_label_6.set_use_markup(True)
-
-        info_label_text_3 = _("<b>Select Workflow Preset</b>")
-        info_label_3 = Gtk.Label(info_label_text_3)
-        info_label_3.set_use_markup(True)
-        guiutils.set_margins(info_label_3, 0, 4, 0, 0)
-            
-        info_label_text_7 = _("You can change and configure individual tools and behaviours <b>anytime</b>")
-        info_label_7 = Gtk.Label(info_label_text_7)
-        info_label_7.set_use_markup(True)
-        
-        info_label_text_4 = _(" by pressing ")
-        info_label_4 = Gtk.Label(info_label_text_4)
-        info_label_4.set_use_markup(True)
-        
-        icon = Gtk.Image.new_from_file(respaths.IMAGE_PATH + "workflow.png")
-    
-        info_label_text_5 = _(" icon.")
-        info_label_5 = Gtk.Label(info_label_text_5)
-        
-        workflow_name = _("<b>Standard</b>")
-        stadard_preset_workflow_text_1 = _("Standard workflow has the <b>Move</b> tool as default tool\nand presents a workflow\nsimilar to most video editors.")
-        workflow_select_item_1 = self.get_workflow_select_item(STANDARD_PRESET, workflow_name, stadard_preset_workflow_text_1)
-
-        workflow_name = _("<b>Film Style</b>")
-        filmstyle_preset_workflow_text_2 = _("Film Style workflow has the <b>Insert</b> tool as default tool\nand employs insert style editing.\nThis was the workflow in previous versions of the application.")
-        workflow_select_item_2 = self.get_workflow_select_item(FILM_STYLE_PRESET, workflow_name, filmstyle_preset_workflow_text_2)
-
-        workflow_name = _("<b>Keep Existing Workflow</b>")
-        keep_workflow_text_2 = _("Select this if you have installed new version and wish to keep your existing workflow.")
-        workflow_select_item_3 = self.get_workflow_select_item(KEEP_EXISTING, workflow_name, keep_workflow_text_2)
-        
-        self.workflow_items = [workflow_select_item_1, workflow_select_item_2, workflow_select_item_3]
-
-        panel_vbox = Gtk.VBox(False, 2)
-        panel_vbox.pack_start(guiutils.get_pad_label(24, 12), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_centered_box([info_label_1]), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_pad_label(24, 12), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_left_justified_box([info_label_2]), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_left_justified_box([info_label_6]), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_pad_label(24, 24), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_centered_box([info_label_3]), False, False, 0)
-        panel_vbox.pack_start(workflow_select_item_1, False, False, 0)
-        panel_vbox.pack_start(workflow_select_item_2, False, False, 0)
-        panel_vbox.pack_start(workflow_select_item_3, False, False, 0)
-        panel_vbox.pack_start(guiutils.get_pad_label(24, 48), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_centered_box([info_label_7]), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_centered_box([info_label_4, icon, info_label_5]), False, False, 0)
-        panel_vbox.pack_start(guiutils.get_pad_label(24, 24), False, False, 0)
-
-        alignment = dialogutils.get_alignment2(panel_vbox)
-
-        self.vbox.pack_start(alignment, True, True, 0)
-        dialogutils.set_outer_margins(self.vbox)
-        dialogs._default_behaviour(self)
-        self.connect('response', self.done)
-        self.show_all()
-
-    def get_workflow_select_item(self, item_number, workflow_name, item_text):
-        name = Gtk.Label(workflow_name)
-        name.set_use_markup(True)
-        guiutils.set_margins(name, 0, 8, 0, 0)
-        label = Gtk.Label(item_text)
-        label.set_use_markup(True)
-        label.set_justify(Gtk.Justification.CENTER)
-
-        item_vbox = Gtk.VBox(False, 2)
-        item_vbox.pack_start(guiutils.get_centered_box([name]), False, False, 0)
-        item_vbox.pack_start(guiutils.get_centered_box([label]), False, False, 0)
-        guiutils.set_margins(item_vbox, 12, 18, 12, 12)
-     
-        widget = Gtk.EventBox()
-        widget.connect("button-press-event", lambda w,e: self.selected_callback(w, item_number))
-        widget.set_can_focus(True)
-        widget.add_events(Gdk.EventMask.KEY_PRESS_MASK)
-
-        widget.add(item_vbox)
-        
-        widget.item_number = item_number
-                
-        self.set_item_color(widget)
-
-        return widget
-
-    def set_item_color(self, widget):
-        if widget.item_number == self.selection:
-            widget.override_background_color(Gtk.StateType.NORMAL, SELECTED_BG)
-            if editorpersistance.prefs.theme == appconsts.LIGHT_THEME:
-                widget.override_color(Gtk.StateType.NORMAL, WHITE_TEXT)
-        else:
-            widget.override_background_color(Gtk.StateType.NORMAL, gui.get_bg_color())
-            if editorpersistance.prefs.theme == appconsts.LIGHT_THEME:
-                widget.override_color(Gtk.StateType.NORMAL, DARK_TEXT)
-
-    def done(self, dialog, response_id):
-        if self.selection == STANDARD_PRESET:
-            _set_workflow_STANDARD()
-        elif self.selection == FILM_STYLE_PRESET:
-            _set_workflow_FILM_STYLE()
-
-        # selection 3, Keep Existing Worflow is just noop
-
-        dialog.destroy()
-
-    def selected_callback(self, w, item_number):
-        self.selection = item_number
-
-        for widget in self.workflow_items:
-            self.set_item_color(widget)
-
